#!/bin/bash

# set -e

: 介绍 <<BLOCK
分析源里的索引文件
BLOCK

EXCUTED=1

function usage() {
    if [ "${EXCUTED}" = 1 ]; then
        echo -e "
USAGE:
    $(basename $0) --help                       # 使用帮助
    $(basename $0) --repo path_to_dists         # 源的索引文件所在目录(dists)
    $(basename $0) --suit suit_name             # 例如：focal focal-security focal-updates
    $(basename $0) --arch amd64|arm64           # 查询的架构
    $(basename $0) --prio required|important    # Priority字段的值
    $(basename $0) --pkg  pkgname               # 要查询的包名

EXAMPLE:
    MIRROR='/data/my_mirror/kylin/dists'
    $(basename $0) --repo \${MIRROR} --pkg  bash
    $(basename $0) --repo \${MIRROR} --prio required
    $(basename $0) --repo \${MIRROR} --suit focal --pkg bash
    $(basename $0) --repo \${MIRROR} --suit v101 --arch amd64 --prio required
    "
        exit 1
    fi
}

if ! $(return >/dev/null 2>&1); then
    # echo "this script is excuted"
    # :
    EXCUTED=1
    # 判断参数个数
    if [ $# -eq 0 ]; then
        usage
    fi
else
    # echo "this script is sourced"
    # :
    EXCUTED=0
fi

function without() {
    # usage:  without "a b c" "a d" -> "b" "c"
    (
        echo "$1" | tr ' ' '\n' | sort | uniq
        echo "$2" "$2" | tr ' ' '\n'
    ) | sort | uniq -u | tr '\n' ' '
    echo
}

function analysis_sigle_file1() {
    FILE=$1
    PRIO=$2

    awk -v SEARCH_PRIO="$PRIO" '
        BEGIN {
            LINO=1;
            PACKAGE_NAME="";
            VERSION="";
            PRIORITY="";
            PREDEPENDS="";
            DEPENDS="";
            FILENAME="";
        }
        {
            if ( $1 ~ "^Package:" ) {
                LINO=NR
                # print NR,$0
                PACKAGE_NAME=$2
            } else if ( $1 ~ "^Version:" ) {
                VERSION=$2
            } else if ( $1 ~ "^Priority:" ) {
                PRIORITY=$2
            } else if ( $1 ~ "^Pre-Depends:" ) {
                sub(/^Pre-Depends: /, "")
                PREDEPENDS=$0
            } else if ( $1 ~ "^Depends:" ) {
                sub(/^Depends: /, "")
                DEPENDS=$0
            } else if ( $1 ~ "^Filename:" ) {
                FILENAME=$2
            } else if ( $1 ~ "^Description:" ) {
                if ( PRIORITY == SEARCH_PRIO ) {
                    # print PACKAGE_NAME
                    print PACKAGE_NAME,VERSION
                    # print DEPENDS
                    # print PACKAGE_NAME"\n"DEPENDS
                    # print PACKAGE_NAME"\n"PREDEPENDS"\n"DEPENDS
                    # print PREDEPENDS"\n"DEPENDS
                    # print LINO,PACKAGE_NAME,PRIORITY,PREDEPENDS,DEPENDS,FILENAME
                }
            }
        }' ${FILE}
}

function analysis_packages1() {
    T=$1

    if [ -n "${SUIT}" ]; then
        T=$1/${SUIT}
    fi

    P=$2
    if [ -d "${T}" ]; then
        Packages_files=$(find ${T} -type d -name binary-${ARCH:-amd64} | xargs -i find "{}" -name Packages)
        if [ -n "${Packages_files}" ]; then
            for i in ${Packages_files}; do
                analysis_sigle_file1 ${i} ${P}
            done
        else
            echo "no 'binary-${ARCH:-amd64}' in '${T}'"
        fi
    elif [ -f "${T}" ]; then
        analysis_sigle_file1 ${T} ${P}
    else
        echo "${T} not exist!!"
    fi
}

function analysis_sigle_file2() {
    match="Package: $2"
    if grep -q "^${match}$" $1; then
        [ -n "${SUIT}" ] || echo "=> $1"
        sed -n "/^${match}$/,/^$/p" $1
    fi
}

function analysis_packages2() {
    T=$1

    if [ -n "${SUIT}" ]; then
        T=$1/${SUIT}
    fi

    P=$2
    if [ -d "${T}" ]; then
        Packages_files=$(find ${T} -type d -name binary-${ARCH:-amd64} | xargs -i find "{}" -name Packages)
        if [ -n "${Packages_files}" ]; then
            for i in ${Packages_files}; do
                analysis_sigle_file2 ${i} ${P}
            done
        else
            echo "no 'binary-${ARCH:-amd64}' in '${T}'"
        fi
    elif [ -f "${T}" ]; then
        analysis_sigle_file2 ${T} ${P}
    else
        echo "${T} not exist!!"
    fi
}

ARGS=$(getopt -a -o h -l help,repo:,suit:,prio:,pkg:,arch: -- "$@")
eval set -- "${ARGS}"

while true; do
    case "$1" in
    -h | --help)
        usage
        shift
        ;;
    --repo)
        TARGET=$2
        shift
        ;;
    --suit)
        SUIT=$2
        shift
        ;;
    --prio)
        case "$2" in
        "required")
            PRIO=$2
            ;;
        "important")
            PRIO=$2
            ;;
        *)
            echo "not support this 'Priority: $2'"
            exit
            ;;
        esac
        shift
        ;;
    --pkg)
        PKGNAME=$2
        shift
        ;;
    --arch)
        ARCH=$2
        shift
        ;;
    --)
        break
        ;;
    esac
    shift
done

: 分析测试 <<BLOCK
source /data/git/gogs/kylin/debootstrap/apt_list_analysis.sh
sudo ln -sf /data/git/gogs/kylin/debootstrap/apt_list_analysis.sh /usr/sbin/apt_list_analysis.sh

# | tr ',' '\n' | tr '|' '\n' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq
# | tr ' ' '\n' | sort | uniq | wc -l
BLOCK

if [ -n "${PRIO}" ]; then
    analysis_packages1 ${TARGET} ${PRIO}
elif [ -n "${PKGNAME}" ]; then
    analysis_packages2 ${TARGET} ${PKGNAME}
else
    usage
fi
