#!/bin/bash

set -e

########## functions ##########
# log
MY_LOG_FILE=/var/tmp/my_analysis.log
: >${MY_LOG_FILE}

function mylog() {
    if [ $# -eq 1 ]; then
        echo "---> $1" >>${MY_LOG_FILE}
    else
        filename=$1
        shift
        echo "$@" >/var/tmp/${filename}
    fi
}

function mylog2() {
    echo "---> $@" >>${MY_LOG_FILE}
}

function do_wget() {
    wget -P ${TARGET_DIR} -r -np $1
    # wget -r -np $1
}

function wgetfile() {
    # echo "$1"
    filepath="${TARGET_DIR}/${1#*//}"
    if [ ! -f "${filepath}" ]; then
        do_wget $1
    else
        mylog2 $(md5sum "${filepath}")
    fi

    if [ "${filepath##*.}" == "gz" ]; then
        if [ ! -f "${filepath%.*}" ]; then
            gunzip -k ${filepath}
        else
            mylog2 $(md5sum "${filepath%.*}")
        fi
    fi
}

function without() {
    # usage:  without "a b c" "a d" -> "b" "c"
    (
        echo "$1" | tr ' ' '\n' | sort | uniq
        echo "$2" "$2" | tr ' ' '\n'
    ) | sort | uniq -u | tr '\n' ' '
    echo
}

# 从 文件1 中排除 文件2
function without2() {
    (
        cat $1
        cat $2
        cat $2
    ) | sort | uniq -u | tr '\n' ' '
}

# 查找依赖 pkgname 的包
function find_who_depend() {
    pkgname="bash"
    result=$(awk -v SEARCH_STR="${pkgname}" -f depends.awk ${path_to_package_file})
    echo "${result}"
}

# 获取依赖
function get_deps() {
    ## 带有 | 的包
    # tr ',' '\n' | tr '|' '\n' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq
    ## 去除 | 后面的包
    # tr ',' '\n' | sed 's/|.*//' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq

    analysis.sh --repo "${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists" --prio required | awk '{print $1}' |
        xargs -i analysis.sh --repo "${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists" --pkg {} |
        grep -E "^Depends:|^Pre-Depends" | sed 's/^.*Depends://' |
        tr ',' '\n' | sed 's/|.*//' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq |
        >/var/tmp/newpkgs.txt1
}

# 单个包的依赖
function get_pkg_deps() {
    . analysis.sh
    pkgname="libpam-runtime"
    analysis_sigle_file2 ${path_to_package_file} ${pkgname} |
        grep -E "^Depends:|^Pre-Depends" | sed 's/^.*Depends://' |
        tr ',' '\n' | sed 's/|.*//' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq
}

# bash包依赖
# resolve_deps2 bash

########## functions ##########

function config_kylin() {
    MIRRORS="http://172.29.157.20/launchpad/kylin http://172.29.157.20/launchpad/kylin-desktop"
    SUITES="v101"
    COMPONENTS="main restricted universe multiverse"
    ARCH="amd64"
}

function config_ubuntu() {
    MIRRORS="http://mirrors.aliyun.com/ubuntu"
    SUITES="focal focal-updates focal-security"
    COMPONENTS="main restricted universe multiverse"
    ARCH="amd64"
}

function config_my() {
    MIRRORS="http://172.29.157.20/my_mirror/kylin"
    SUITES="v101"
    COMPONENTS="main"
    ARCH="amd64"
    TARGET_DIR="/var/tmp/download"
    if [ ! -d "${TARGET_DIR}" ]; then
        mkdir -p "${TARGET_DIR}"
    fi
    TARGET="/var/tmp/rootfs"
    if [ ! -d "${TARGET}" ]; then
        sudo mkdir -p "${TARGET}"
    fi
}

function download_indices() {
    for m in ${MIRRORS}; do
        for s in ${SUITES}; do
            sInRelease="${m}/dists/${s}/InRelease"
            sRelease="${m}/dists/${s}/Release"
            sRelease_gpg="${m}/dists/${s}/Release.gpg"

            wgetfile ${sInRelease}
            wgetfile ${sRelease}
            wgetfile ${sRelease_gpg}

            package_urls=""
            for c in ${COMPONENTS}; do
                url="${m}/dists/$s/$c/binary-$ARCH/Packages.gz"
                # echo ${url}
                package_urls="${url} ${package_urls}"
                wgetfile ${url}
            done
        done
    done
}

function url_to_path() {
    filepath="${TARGET_DIR}/${1#*//}"
    echo ${filepath%.*}
}

function resolve_deps2() {
    local PKGS="$*"
    local ALLPKGS="$PKGS"
    local ALLPKGS2=""

    while [ "$PKGS" != "" ]; do
        # 查找依赖包
        local NEWPKGS=$(
            for p in ${package_urls}; do
                for i in $(echo "${PKGS}" | tr ' ' '\n'); do
                    match="Package: ${i}"
                    if grep -q "^${match}$" "$(url_to_path ${p})"; then
                        sed -n "/^${match}$/,/^$/p" "$(url_to_path ${p})"
                    fi
                done
            done | grep -E "^Depends:|^Pre-Depends" | sed 's/^.*Depends://' | tr ',' '\n' | sed 's/|.*//' | sed 's#(.*)##g' | xargs | tr ' ' '\n' | sort | uniq
        )
        mylog "newpkgs.txt2" "$(echo ${NEWPKGS} | xargs | tr ' ' '\n')"

        PKGS=$(echo "$PKGS $NEWPKGS" | tr ' ' '\n' | sort | uniq)

        # 真实的包
        local REALPKGS=""
        for p in ${package_urls}; do
            for i in $PKGS; do
                if grep -q "^Package: ${i}$" "$(url_to_path ${p})"; then
                    # echo ${i}
                    if [ -n "${REALPKGS}" ]; then
                        REALPKGS="${REALPKGS} ${i}"
                    else
                        REALPKGS="${i}"
                    fi
                fi
            done
        done

        mylog "realpkgs.txt2" "$(echo ${REALPKGS} | xargs | tr ' ' '\n')"

        PKGS="$REALPKGS"
        ALLPKGS2=$(echo "$PKGS $ALLPKGS" | tr ' ' '\n' | sort | uniq)
        PKGS=$(without "$ALLPKGS2" "$ALLPKGS")

        ALLPKGS="$ALLPKGS2"
    done
    echo "$ALLPKGS"
}

function work_out_debs() {
    PRIO="required"
    required=$(
        for p in ${package_urls}; do
            awk -v SEARCH_STR="${PRIO}" -f ${SHELL_DIR}/analysis.awk "$(url_to_path ${p})"
        done
    )
    mylog "01-required.txt2" "${required}"
    required_deps=$(resolve_deps2 "${required}")
    mylog "01-required@deps.txt2" "$(echo ${required_deps} | xargs | tr ' ' '\n')"

    PRIO="important"
    important=$(
        for p in ${package_urls}; do
            awk -v SEARCH_STR="${PRIO}" -f ${SHELL_DIR}/analysis.awk "$(url_to_path ${p})"
        done
    )
    mylog "02-important.txt2" "${important}"
    important_deps=$(resolve_deps2 "${important}")
    mylog "02-important@deps.txt2" "$(echo ${important_deps} | xargs | tr ' ' '\n')"
    important_deps=$(without "$important_deps" "$required_deps")
    mylog "02-important@deps@without@required.txt2" "$(echo ${important_deps} | xargs | tr ' ' '\n')"
}

function download_debs() {
    ALL_PKGS="$*"
    MIRROR=${MIRRORS%% *}
    for p in ${package_urls}; do
        for i in ${ALL_PKGS}; do
            match="Package: ${i}"
            filename=$(
                if grep -q "^${match}$" "$(url_to_path ${p})"; then
                    sed -n "/^${match}$/,/^$/p" "$(url_to_path ${p})"
                fi | grep '^Filename: ' | awk '{print $2}'
            )
            # filename="${MIRROR}/$(analysis.sh --repo ${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists --pkg $i | grep '^Filename: ' | awk '{print $2}')"
            echo "$i $filename"
            wgetfile ${MIRROR}/$filename
        done
    done | column -t >${TARGET_DIR}/deb.list
}

debfor2() {
    for p in ${package_urls}; do
        p=${p#*//}
        p=${p%dists*}
        for i in "$@"; do
            local filename=$(grep "^${i} " ${TARGET_DIR}/deb.list | awk '{print $2}')
            echo "${TARGET_DIR}/${p}/${filename}"
        done
    done
}

function my_first_stage_install() {

    # 先删除目录
    sudo rm -rf $TARGET

    setup_merged_usr() {
        link_dir="bin sbin lib lib32 lib64 libx32"
        local dir
        for dir in $link_dir; do
            mkdir -p "$TARGET/usr/$dir"
            ln -sf usr/"$dir" "$TARGET/$dir"
        done
    }

    # Create a device node if it does not exist. By default, the mode is 666.
    mknod_if_needed() {
        local device type major minor mode
        device="$1"
        type="$2"
        major="$3"
        minor="$4"
        mode="${5:-666}"

        if [ ! -e "$device" ]; then
            mknod -m "$mode" "$device" "$type" "$major" "$minor"
        fi
    }

    setup_devices_simple() {
        # The list of devices that can be created in a container comes from
        # src/core/cgroup.c in the systemd source tree.
        mknod_if_needed "$TARGET/dev/null" c 1 3
        mknod_if_needed "$TARGET/dev/zero" c 1 5
        mknod_if_needed "$TARGET/dev/full" c 1 7
        mknod_if_needed "$TARGET/dev/random" c 1 8
        mknod_if_needed "$TARGET/dev/urandom" c 1 9
        mknod_if_needed "$TARGET/dev/tty" c 5 0
        if [ ! "$CONTAINER" = "systemd-nspawn" ]; then
            mknod_if_needed "$TARGET/dev/console" c 5 1
        fi
        # To avoid pre-exist directory causes error, specify "-p" option
        mkdir -p "$TARGET/dev/pts/" "$TARGET/dev/shm/"
        # Inside a container, we might not be allowed to create /dev/ptmx.
        # If not, do the next best thing.
        if ! mknod_if_needed "$TARGET/dev/ptmx" c 5 2; then
            warning MKNOD "Could not create /dev/ptmx, falling back to symlink. This chroot will require /dev/pts mounted with ptmxmode=666"
            ln -sf pts/ptmx "$TARGET/dev/ptmx"
        fi
        ln -sf /proc/self/fd "$TARGET/dev/fd"
        ln -sf /proc/self/fd/0 "$TARGET/dev/stdin"
        ln -sf /proc/self/fd/1 "$TARGET/dev/stdout"
        ln -sf /proc/self/fd/2 "$TARGET/dev/stderr"
    }

    conditional_cp() {
        if [ ! -e "$2/$1" ]; then
            if [ -L "$1" ] && [ -e "$1" ]; then
                cat "$1" >"$2/$1"
            elif [ -e "$1" ]; then
                cp "$1" "$2/$1"
            fi
        fi
    }

    setup_merged_usr

    debfor2 $required_deps | xargs -i bash -c "dpkg-deb --fsys-tarfile {} | tar -k -xvf - -C $TARGET"

    mkdir -p "$TARGET/var/lib/dpkg"
    : >"$TARGET/var/lib/dpkg/status"
    : >"$TARGET/var/lib/dpkg/available"

    conditional_cp /etc/resolv.conf "$TARGET"
    conditional_cp /etc/hostname "$TARGET"

    if [ ! -e "$TARGET/etc/fstab" ]; then
        echo '# UNCONFIGURED FSTAB FOR BASE SYSTEM' >"$TARGET/etc/fstab"
        chown 0:0 "$TARGET/etc/fstab"
        chmod 644 "$TARGET/etc/fstab"
    fi

    setup_devices_simple

    # setup_apt_sources
    echo "deb ${MIRRORS} ${SUITES} ${COMPONENTS}" >"$TARGET/etc/apt/sources.list"

    # set alias chroot ls --color
    sudo chroot $TARGET bash -c "echo \"alias ls='ls --color'\" >> ~/.bashrc"
}

function my_second_stage_install() {
    # 拷贝deb到rootfs
    [ -d $TARGET/var/tmp/download ] || sudo cp -rf ${TARGET_DIR} $TARGET/var/tmp

    in_target() {
        sudo chroot $TARGET $*
    }

    x_feign_install() {
        local pkg="$1"
        local deb="$(debfor2 $pkg)"
        local ver="$(in_target dpkg-deb -f "$deb" Version)"

        echo "Package: $pkg
Version: $ver
Maintainer: unknown
Status: install ok installed
" >"$TARGET/var/lib/dpkg/status"
        # touch "$TARGET/var/lib/dpkg/info/${pkg}.list"
        : >$TARGET/var/lib/dpkg/info/${pkg}.list
    }

    x_feign_install dpkg

    x_core_install() {
        mylog2 in_target dpkg --force-depends --install $(debfor2 "$@")
        yes | in_target dpkg --force-depends --install $(debfor2 "$@")
    }

    p() {
        baseprog="$(($baseprog + ${1:-1}))"
    }

    # setup_proc
    sudo mount -t proc proc $TARGET/proc
    sudo mount -t sysfs sysfs $TARGET/sys
    in_target /sbin/ldconfig

    DEBIAN_FRONTEND=noninteractive
    DEBCONF_NONINTERACTIVE_SEEN=true
    export DEBIAN_FRONTEND DEBCONF_NONINTERACTIVE_SEEN

    echo "Installing core packages..."

    ln -sf mawk "$TARGET/usr/bin/awk"

    x_core_install base-passwd
    x_core_install base-files
    x_core_install dpkg

    if [ ! -e "$TARGET/etc/localtime" ]; then
        ln -sf /usr/share/zoneinfo/UTC "$TARGET/etc/localtime"
    fi

    LIBC="libc6"
    x_core_install $LIBC

    x_core_install perl-base

    rm "$TARGET/usr/bin/awk"
    x_core_install mawk

    # if doing_variant -; then
    x_core_install debconf

    echo "Unpacking required packages..."

    # TODO: repeat some times
    # exec 7>&1
    # yes | in_target in_targetdpkg --status-fd 8 --force-depends --unpack $(debfor2 $required_deps) 8>&1 1>&7
    yes | in_target dpkg --force-depends --unpack $(debfor2 $required_deps)

    echo "Configuring required packages..."

    echo "#!/bin/sh
    exit 101" >"$TARGET/usr/sbin/policy-rc.d"
    chmod 755 "$TARGET/usr/sbin/policy-rc.d"

    mv "$TARGET/sbin/start-stop-daemon" "$TARGET/sbin/start-stop-daemon.REAL"

    echo "#!/bin/sh
    echo
    echo \"Warning: Fake start-stop-daemon called, doing nothing\"" >"$TARGET/sbin/start-stop-daemon"

    chmod 755 "$TARGET/sbin/start-stop-daemon"

    echo "apt apt" >"$TARGET/var/lib/dpkg/cmethopt"
    chmod 644 "$TARGET/var/lib/dpkg/cmethopt"

    # yes | in_target dpkg --status-fd 8 --configure --pending --force-configure-any --force-depends 8>&1 1>&7
    yes | in_target dpkg --configure --pending --force-configure-any --force-depends

    echo "Unpacking the base system..."

    # setup_available
    for p in ${package_urls}; do
        for i in ${required_deps} ${important_deps}; do
            sed -n "/^Package: ${i}$/,/^$/p" "$(url_to_path ${p})"
        done
    done >"$TARGET/var/lib/dpkg/available"

    get_next_predep() {
        local stanza="$(in_target dpkg --predep-package)"
        [ "$stanza" ] || return 1
        echo "$stanza" | grep '^Package:' | sed 's/^Package://; s/^ *//'
    }

    done_predeps=
    while predep=$(get_next_predep); do
        # We have to resolve dependencies of pre-dependencies manually because dpkg --predep-package doesn't handle this.
        predep=$(without "$(without "$(resolve_deps2 $predep)" "$required")" "$done_predeps")
        yes | in_target dpkg --force-overwrite --force-confold --skip-same-version --install $(debfor2 $predep)
        base=$(without "$base" "$predep")
        done_predeps="$done_predeps $predep"
    done

    if [ -n "$base" ]; then
        # yes | in_target dpkg --status-fd 8 --force-overwrite --force-confold --skip-same-version --unpack $(debfor2 $base) 8>&1 1>&7
        yes | in_target dpkg --force-overwrite --force-confold --skip-same-version --unpack $(debfor2 $base)

        echo "Configuring the base system..."

        # yes | in_target dpkg --status-fd 8 --force-confold --skip-same-version --configure -a 8>&1 1>&7
        yes | in_target dpkg --force-confold --skip-same-version --configure -a
    fi

    mv "$TARGET/sbin/start-stop-daemon.REAL" "$TARGET/sbin/start-stop-daemon"
    rm -f "$TARGET/usr/sbin/policy-rc.d"

    echo "Configuring base system"
    echo "Base system installed successfully."

    # rm -rf $TARGET/var/tmp/download
}

SHELL_DIR=$(dirname $(readlink -f $0))

trap "exit_function" 0

exit_function() {
    if mountpoint -q $TARGET/proc; then
        sudo umount $TARGET/proc
    fi

    if mountpoint -q $TARGET/sys; then
        sudo umount $TARGET/sys
    fi
    echo "exit_function" >/dev/null
}

# config_ubuntu
# config_kylin
config_my

download_indices
# work_out_debs
# all_debs=$(echo "${required_deps} ${important_deps}" | xargs | tr ' ' '\n' | sort | uniq)
# # echo "${all_debs}"
# download_debs "${all_debs}"

required_deps="$(cat /var/tmp/01-required@deps.txt2)"
important_deps="$(cat /var/tmp/02-important@deps@without@required.txt2)"

my_first_stage_install
my_second_stage_install
