#!/bin/bash

# set -e

: 介绍 <<BLOCK
分析源索引测试
BLOCK

EXCUTED=1

function usage() {
    if [ "${EXCUTED}" = 1 ]; then
        echo -e "
USAGE:
    $(basename $0) --help                       # 使用帮助
    $(basename $0) --type u|l|ld|k|m            # 分析类型
        u:ubuntu                Ubuntu标准源
        l:launchpad             launchpad基础源
        ld:launchpad-desktop    launchpad-desktop
        k:KYLIN-ALL             V10 SP1 桌面源
        m:my_mirror             基于桌面源 自建的源
    $(basename $0) --arch amd64|arm64           # 查询的架构
    $(basename $0) --prio required|important    # Priority字段的值
    $(basename $0) --pkg  pkgname               # 要查询的包名

EXAMPLE:
    $(basename $0) --type u --pkg  bash
    $(basename $0) --type u --prio required
    $(basename $0) --type u --suit focal --pkg bash
    $(basename $0) --type u --suit v101 --arch amd64 --prio required
    "
        exit 1
    fi
}

if ! $(return >/dev/null 2>&1); then
    # echo "this script is excuted"
    # :
    EXCUTED=1
    # 判断参数个数
    if [ $# -eq 0 ]; then
        usage
    fi
else
    # echo "this script is sourced"
    # :
    EXCUTED=0
fi

# 合并launchpad
function merge_launchpad() {
    # 去除同时在launchpad-required.list 和 launchpad-desktop-required.list 的包
    awk '{print $1}' launchpad-desktop-required.list | xargs -i sed -i '/^{} */d' launchpad-required.list
    cat launchpad-required.list launchpad-desktop-required.list | sort >launchpad-all-required.list
}

function compare_two_version() {
    echo "-> only in $1"
    for i in $(awk '{print $1}' $1); do
        if ! grep -q "^${i} *" $2; then
            sed -n "/^${i} */p" $1
        fi
    done

    echo "-> only in $2"
    for i in $(awk '{print $1}' $2); do
        if ! grep -q "^${i} *" $1; then
            sed -n "/^${i} */p" $2
        fi
    done
}

# 查找某一个包
function find_pkg() {
    # MIRROR="/data/mirror/launchpad/mirror/archive.launchpad.dev/kylin/dists"
    # MIRROR="/data/mirror/163/ubuntu-20.04/mirror/mirrors.163.com/ubuntu/dists"

    MIRROR="/data/mirror/KYLIN-ALL/mirror/archive.kylinos.cn/kylin/KYLIN-ALL/dists"
    apt_list_analysis.sh --repo ${MIRROR} --suit 10.1 --arch amd64 --pkg $1
}

# my_test required
# my_test important
# find_pkg libselinux1

# compare_two_version launchpad-all-required.list KYLIN-ALL-required.list
# libpam0g                1.3.1-5kylin4k1
# libselinux1             3.0-1build2kylin0k

# compare_two_version launchpad-required.list my_mirror-required.list
# -> only in launchpad-required.list
# libacl1                 2.2.53-6
# libattr1                1:2.4.48-5
# libgcc1                 1:10-20200411-0kylin1
# liblzma5                5.2.4-1kylin2
# libpam0g                1.3.1-5kylin4k1
# libselinux1             3.0-1build2kylin0k1
# libsepol1               3.0-2
# lzma                    9.22-2.1build1
# xz-utils                5.2.4-1kylin2
# -> only in my_mirror-required.list
# gcc-7-base              7.5.0-6kylin2
# gcc-8-base              8.4.0-3kylin2
# kylin-apt-keyring       2022.04.12
# libsecurity1            1.1-17kylin
# libsecurity1-scripts    1.1-17kylin
# libsecurity1-scripts    1.1-17kylin
# libsecurity-conf        1.1-17kylin
# libsecurity-conf-dev    1.1-17kylin

ARGS=$(getopt -a -o h -l help,type:,prio:,pkg:,arch: -- "$@")
eval set -- "${ARGS}"

while true; do
    case "$1" in
    -h | --help)
        usage
        shift
        ;;
    --type)
        case "$2" in
        "u")
            # Ubuntu标准源
            TYPE="ubuntu"
            S="focal"
            MIRROR="/data/mirror/163/ubuntu-20.04/mirror/mirrors.163.com/ubuntu/dists"
            ;;
        "l")
            TYPE="launchpad"
            S="v101"
            MIRROR="/data/mirror/launchpad/mirror/archive.launchpad.dev/kylin/dists"
            ;;
        "ld")
            TYPE="launchpad-desktop"
            S="v101"
            MIRROR="/data/mirror/launchpad/mirror/archive.launchpad.dev/kylin-desktop/dists"
            ;;
        "k")
            TYPE="KYLIN-ALL"
            S="10.1"
            MIRROR="/data/mirror/KYLIN-ALL/mirror/archive.kylinos.cn/kylin/KYLIN-ALL/dists"
            ;;
        "m")
            TYPE="my_mirror"
            S="v101"
            MIRROR="/data/my_mirror/kylin/dists"
            ;;
        *)
            echo "not support type: $2"
            exit
            ;;
        esac
        shift
        ;;
    --suit)
        SUIT=$2
        shift
        ;;
    --prio)
        case "$2" in
        "required")
            PRIO=$2
            ;;
        "important")
            PRIO=$2
            ;;
        *)
            echo "not support this 'Priority: $2'"
            exit
            ;;
        esac
        shift
        ;;
    --pkg)
        PKGNAME=$2
        shift
        ;;
    --arch)
        ARCH=$2
        shift
        ;;
    --)
        break
        ;;
    esac
    shift
done

: 分析测试 <<BLOCK
source /data/git/gogs/kylin/debootstrap/apt_list_test.sh
sudo ln -sf /data/git/gogs/kylin/debootstrap/apt_list_test.sh /usr/sbin/apt_list_test.sh

# 分析单一类型
apt_list_test.sh --type m --prio required
apt_list_test.sh --type m --pkg bash

# 分析全部类型
for type in "u l ld k m"
do
    echo ${type}
    apt_list_test.sh --type {type} --prio required
done
BLOCK

echo "MIRROR=${MIRROR} SUIT=${SUIT} ARCH=${ARCH} PRIO=${PRIO} PKGNAME=${PKGNAME}"

if [ -n "${PRIO}" ]; then
    apt_list_analysis.sh --repo ${MIRROR} --suit ${SUIT} --arch ${ARCH:-amd64} --prio ${PRIO} | sort | column -t >${TYPE}-${PRIO}.list
elif [ -n "${PKGNAME}" ]; then
    apt_list_analysis.sh --repo ${MIRROR} --suit ${SUIT} --arch ${ARCH:-amd64} --pkg ${PKGNAME}
else
    usage
fi
