#################### debian ####################
function debian() {
    sudo debootstrap buster buster-rootfs http://ftp.cn.debian.org/debian/
    sudo debootstrap --verbose buster buster-rootfs http://mirrors.aliyun.com/debian

    sudo debootstrap --arch=arm64 --foreign stretch linux-rootfs http://ftp.cn.debian.org/debian/

    # 打印debs
    sudo debootstrap --print-debs buster buster-rootfs http://ftp.cn.debian.org/debian/

    # --download-only
    sudo debootstrap --verbose --download-only --variant=minbase buster buster-rootfs http://ftp.cn.debian.org/debian/
}

#################### ubuntu ####################
function ubuntu() {
    # MIRROR_URL="file:///data/mirror/163/ubuntu-20.04/mirror/mirrors.163.com/ubuntu"

    # 尝试多源的情况，并不好使
    # MIRROR_URL="http://mirrors.aliyun.com/ubuntu http://mirrors.163.com/ubuntu"

    MIRROR_URL="http://mirrors.163.com/ubuntu"

    sudo debootstrap --no-check-gpg --verbose --keep-debootstrap-dir --variant=minbase focal focal-rootfs "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --verbose --print-debs --variant=minbase focal focal-rootfs "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --verbose --print-debs --keep-debootstrap-dir --variant=minbase focal focal-rootfs "${MIRROR_URL}"

    # ubuntu 22.04
    MIRROR_URL="http://mirror.xiayishusheng.com/ubuntu"
    sudo debootstrap --no-check-gpg --verbose --keep-debootstrap-dir --variant=minbase jammy jammy-rootfs "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --verbose --print-debs --keep-debootstrap-dir --variant=minbase jammy jammy-rootfs "${MIRROR_URL}"
}

#################### kylin: KYLIN-ALL 这个源会报错 ####################
# 因为bash包中的postinst是二进制文件，dpkg处理时在文件头部添加了#!/bin/bash导致失败。
# 自建源，修改postinst解决。
function kylin_10.1() {
    # 创建 10.1 软链接
    sudo ln -sf gutsy /usr/share/debootstrap/scripts/10.0
    sudo ln -sf gutsy /usr/share/debootstrap/scripts/v100

    # deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main universe multiverse restricted

    ROOTFS_DIR=KYLIN-ALL-rootfs
    # MIRROR_URL="http://archive.kylinos.cn/kylin/KYLIN-ALL"
    # MIRROR_URL="http://172.29.157.20/mirror/juniper"
    MIRROR_URL="http://172.29.220.247:8000/" # 左侧大台式机上同步的v10版本的源

    sudo debootstrap --no-check-gpg --print-debs --keep-debootstrap-dir --components=main,restricted,universe,multiverse 10.0 ${ROOTFS_DIR} "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --components=main,restricted,universe,multiverse 10.0 ${ROOTFS_DIR} "${MIRROR_URL}"
}

#################### kylin v10 ####################
function kylin_10.0() {
    # 创建 10.1 软链接
    sudo ln -sf gutsy /usr/share/debootstrap/scripts/10.0

    # deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.0 main universe multiverse restricted

    ROOTFS_DIR=KYLIN-ALL-rootfs
    # MIRROR_URL="http://archive.kylinos.cn/kylin/KYLIN-ALL"
    MIRROR_URL="http://172.29.157.20/kylin/KYLIN-ALL"

    sudo debootstrap --no-check-gpg --print-debs --keep-debootstrap-dir --components=main,restricted,universe,multiverse 10.1 ${ROOTFS_DIR} "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --components=main,restricted,universe,multiverse 10.1 ${ROOTFS_DIR} "${MIRROR_URL}"
}

#################### kylin: launchpad.dev ####################
function kylin_launchpad() {
    # 非麒麟系统，创建 v101 软链接
    if [ ! "$(lsb_release -i | awk '{print $NF}')" = "Kylin" ]; then
        sudo ln -sf gutsy /usr/share/debootstrap/scripts/v101
    else
        echo "This is Kylin OS, don't need create v101"
    fi

    # rootfs 路径
    ROOTFS_DIR=rootfs-kylin-amd64

    # MIRROR_URL="file:///data/mirror/launchpad/kylin/mirror/archive.launchpad.dev/kylin"

    # MIRROR_URL="http://archive.launchpad.dev/kylin/"
    # MIRROR_URL="http://archive.launchpad.dev/kylin-desktop/"

    # 本地launchpad
    # MIRROR_URL="http://172.29.157.20/launchpad/kylin"
    # MIRROR_URL="http://172.29.157.20/launchpad/kylin-desktop"
    # MIRROR_URL="http://172.29.157.20/launchpad/kylin http://172.29.157.20/launchpad/kylin-desktop"
    # MIRROR_URL="http://172.29.157.20/launchpad/kylin-desktop http://172.29.157.20/launchpad/kylin"

    # 麒麟桌面源下载deb包，自建源，改了bash，添加gpg文件
    MIRROR_URL="http://172.29.157.20/my_mirror/kylin"

    [ -d ${ROOTFS_DIR} ] && sudo rm -rf ${ROOTFS_DIR}

    # --print-debs
    sudo debootstrap --no-check-gpg --print-debs --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --print-debs --keep-debootstrap-dir
    sudo debootstrap --no-check-gpg --print-debs --keep-debootstrap-dir --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --variant=minbase
    sudo debootstrap --no-check-gpg --print-debs --keep-debootstrap-dir --variant=minbase --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # default
    sudo debootstrap --no-check-gpg --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --variant=minbase
    sudo debootstrap --no-check-gpg --variant=minbase --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --cache-dir
    CACHE_DIR="cached-debs"
    mkdir -p ${CACHE_DIR}
    sudo debootstrap --no-check-gpg --variant=minbase --components=main,restricted,universe,multiverse --cache-dir=$(realpath ${CACHE_DIR}) v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # --download-only
    sudo debootstrap --no-check-gpg --download-only --keep-debootstrap-dir --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # --foreign(第一阶段)
    sudo debootstrap --no-check-gpg --foreign --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --second-stage(第二阶段)
    sudo chroot ${ROOTFS_DIR} /debootstrap/debootstrap --second-stage

    # --make-tarball
    ROOTFS_TAR="rootfs.tar.gz"
    sudo debootstrap --no-check-gpg --make-tarball=${ROOTFS_TAR} --keep-debootstrap-dir --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # --unpack-tarball
    sudo debootstrap --no-check-gpg --unpack-tarball=$(realpath ${ROOTFS_TAR}) --components=main,restricted,universe,multiverse v101 ${ROOTFS_DIR} "${MIRROR_URL}"
}

# 使用长城笔记本制作 arm64 版本rootfs
function changcheng_arm64() {
    ROOTFS_DIR="rootfs-kylin-arm64"
    MIRROR_URL="http://172.29.157.20/my_mirror/kylin"

    ## 包含 resize-assistant, 来自 rock3a
    DEBS_RESIZE="resize-assistant"
    ## tools（可选）
    DEBS_TOOLS="tmux,make,vim,bash-completion,dfc"

    ## 语言相关包
    DEBS_LANGUAGE="locales,language-pack-zh-hans"

    ## 安装网络相关包
    DEBS_NETWORK="network-manager,netplan.io,iputils-ping,iproute2,net-tools"
    ## 安装网络及其他相关包 # python3-cffi-backend；dialog: 安装包时，debconf 需要的交互式前端
    DEBS_DEPDENDS="python3-cffi-backend,dialog"

    ## xserver
    DEBS_XSERVER="xinit,xserver-xorg,xterm,openbox"

    ## kylin-os-installer,xmodmap: x11-xserver-utils, /usr/lib/udisks2/udisks2-inhibit: udisks2
    # DEBS_KYLINOS_INSTALLER="kylin-os-installer,lightdm,x11-xserver-utils,udisks2,sudo"
    DEBS_KYLINOS_INSTALLER="kylin-os-installer"

    # 各种组合
    DEBS_INCLUDE="${DEBS_RESIZE}"
    DEBS_INCLUDE="${DEBS_RESIZE},${DEBS_TOOLS}"
    DEBS_INCLUDE="${DEBS_RESIZE},${DEBS_TOOLS},${DEBS_LANGUAGE},${DEBS_NETWORK},${DEBS_DEPDENDS}"
    DEBS_INCLUDE="${DEBS_RESIZE},${DEBS_TOOLS},${DEBS_LANGUAGE},${DEBS_NETWORK},${DEBS_DEPDENDS},${DEBS_XSERVER}"
    DEBS_INCLUDE="${DEBS_RESIZE},${DEBS_TOOLS},${DEBS_LANGUAGE},${DEBS_NETWORK},${DEBS_DEPDENDS},${DEBS_KYLINOS_INSTALLER}"

    # --keep-debootstrap-dir
    sudo debootstrap --no-check-gpg --keep-debootstrap-dir --arch=arm64 --include=${DEBS_INCLUDE} --components=main v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    sudo debootstrap --no-check-gpg --arch=arm64 --include=${DEBS_INCLUDE} --components=main v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # 默认, --variant=minbase 不会安装 ubuntukylin-keyring
    sudo debootstrap --no-check-gpg --variant=minbase --arch=arm64 --components=main v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # 配置 kylin 用户
    USERNAME="kylin"
    PASSWORD="qwer1234"
    useradd -m -s /bin/bash -u 1000 "${USERNAME}"
    usermod -c "${USERNAME}" "${USERNAME}"
    echo "${USERNAME}:${PASSWORD}" | chpasswd
    adduser ${USERNAME} sudo
    adduser ${USERNAME} adm

    # 最小版本 minbase
    sudo debootstrap --no-check-gpg --arch=arm64 --variant=minbase --components=main v101 ${ROOTFS_DIR} "${MIRROR_URL}"
    # 包含 gpg,wget,gpg-agent
    sudo debootstrap --no-check-gpg --arch=arm64 --include=gpg,wget,gpg-agent --components=main v101 ${ROOTFS_DIR} "${MIRROR_URL}"

    # 删除rootfs目录/var/cache/apt/下的deb包
    sudo rm -rf ${ROOTFS_DIR}/var/cache/apt/archives/*.deb
    sudo rm -rf ${ROOTFS_DIR}/var/cache/apt/*.bin
    sudo rm -rf ${ROOTFS_DIR}/var/lib/apt/lists/*_Packages
    sudo rm -rf ${ROOTFS_DIR}/var/lib/apt/lists/*_InRelease
    sudo rm -rf ${ROOTFS_DIR}/root/.bash_history

    # 制作sdcard镜像
    dd if=/dev/zero of=boot.disk bs=1M count=512
    # EFI 类型为 EF00
    sgdisk -n 0:0:+80M -c 0:boot -t 0:EF00 boot.disk
    sgdisk -n 0:0:0 -c 0:rootfs boot.disk
}

#################### arm 架构 ####################
function arm() {
    # qemu-debootstrap 参数和 debootstrap 一样
    sudo qemu-debootstrap xxxxxx

    # deb http://ports.ubuntu.com/ubuntu-ports trusty main
    MIRROR_URL="http://ports.ubuntu.com/ubuntu-ports"

    # ubuntu 18.04
    sudo qemu-debootstrap --verbose --arch=arm64 --variant=minbase bionic bionic-rootfs "${MIRROR_URL}"
    # ubuntu 20.04
    sudo qemu-debootstrap --no-check-gpg --verbose --arch=arm64 --variant=minbase focal focal-rootfs "${MIRROR_URL}"
    # ubuntu 14.04
    sudo qemu-debootstrap --no-check-gpg --verbose --arch=armhf --variant=minbase trusty trusty-rootfs/
    # debian stretch
    MIRROR_URL="http://mirrors.ustc.edu.cn/raspbian/raspbian"
    sudo debootstrap --arch=arm64 --foreign stretch linux-rootfs "${MIRROR_URL}"

    # --print-debs
    sudo debootstrap --no-check-gpg --print-debs --arch=arm64 --components=main,restricted,universe,multiverse v101 kylin-arm64-rootfs http://archive.launchpad.dev/kylin gutsy
    sudo debootstrap --no-check-gpg --print-debs --arch=arm64 --components=main,restricted,universe,multiverse v101 kylin-arm64-rootfs "${MIRROR_URL}"

    # --foreign arm64
    sudo debootstrap --no-check-gpg --arch=arm64 --foreign --components=main,restricted,universe,multiverse v101 kylin-arm64-rootfs http://archive.launchpad.dev/kylin gutsy
    sudo debootstrap --no-check-gpg --arch=arm64 --components=main,restricted,universe,multiverse v101 kylin-arm64-rootfs http://archive.launchpad.dev/kylin gutsy
    sudo debootstrap --no-check-gpg --arch=arm64 --components=main,restricted,universe,multiverse v101 kylin-arm64-rootfs "${MIRROR_URL}"
}

#################### rpi ####################
function rpi() {
    source isar-init-build-env ../rpi
    bitbake mc:rpi-stretch:isar-image-base

    # 报错信息
    # Tried to extract package

    # | I: Running command: debootstrap --arch armhf --foreign --verbose --variant=minbase --include=locales,gnupg --keyring=/home/zhen/rpi/tmp/work/raspbian-stretch-armhf/isar-bootstrap-target/1.0-r0/distro-keyring.gpg --no-check-gpg --components=main,contrib,non-free,firmware stretch /home/zhen/rpi/tmp/work/raspbian-stretch-armhf/isar-bootstrap-target/1.0-r0/rootfs http://mirrors.ustc.edu.cn/raspbian/raspbian

    sudo /usr/sbin/debootstrap --arch armhf --foreign --verbose --variant=minbase --include=locales --components=main,contrib,non-free bullseye ./rpi-rootfs http://deb.debian.org/debian
    sudo /usr/sbin/debootstrap --arch armhf --foreign --verbose --variant=minbase --include=locales --components=main,contrib,non-free stretch ./rpi-rootfs http://mirrors.ustc.edu.cn/raspbian/raspbian

    # armhf
    # E: Couldn't download http://archive.launchpad.dev/kylin/dists/v101/restricted/binary-armhf/Packages
}

# 两种解 deb 的方式
function extract_deb() {
    # 方法1
    for i in $(ls /data/temp/kylin-rootfs/var/cache/apt/archives/*.deb); do
        echo $i
        dpkg-deb --fsys-tarfile $i | tar -k -xf -
    done

    # 方法2
    for i in $(ls /data/temp/kylin-rootfs/var/cache/apt/archives/*.deb); do
        echo $i
        dpkg-deb -x $i .
    done

    # 解压deb到 my_rootfs
    for i in $(ls *.deb); do
        echo $i
        dpkg-deb -x "${i}" /data/temp/my_rootfs
    done
}

# 将 rootfs 目录中的内容打包成tar包，並导入到Docker镜像列表中
function docker() {
    # 方法1
    sudo tar cvf kylin-rootfs.tar -C kylin-rootfs .
    docker import kylin-rootfs.tar kylin

    # 方法2
    sudo tar c -C kylin-rootfs . | docker import - kylin

    # 运行
    docker run -it --rm kylin /bin/bash

    ## rootfs-kylin-amd64
    sudo tar cvf rootfs-kylin-amd64.tar -C rootfs-kylin-amd64 .
    docker import rootfs-kylin-amd64.tar rootfs-kylin-amd64

    docker run -it --name rootfs-kylin-amd64 --hostname rootfs-kylin-amd64 -v /data:/data -d rootfs-kylin-amd64:20220705 bash
    docker start rootfs-kylin-amd64
    docker exec -it rootfs-kylin-amd64 bash

    ## rootfs-kylin-arm64
    sudo tar cvf rootfs-kylin-arm64.tar -C rootfs-kylin-arm64 .
    docker import rootfs-kylin-arm64.tar rootfs-kylin-arm64

    docker run -it --name rootfs-kylin-arm64 --hostname rootfs-kylin-arm64 -v /data:/data -d rootfs-kylin-arm64:20220705 bash
    docker start rootfs-kylin-arm64
    docker exec -it rootfs-kylin-arm64 bash
}
