#!/bin/bash
set -e

export HOME=/root
export LC_ALL=C

cat >/etc/apt/sources.list <<EOF
deb http://172.29.157.20/my_mirror/kylin v101 main
EOF

apt update

# set background
apt install -y simple-wallpaper

# ==================== create system ====================
# 制作过程中使用的源
cat >/etc/apt/sources.list <<EOF
deb http://172.29.157.20/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
EOF

apt update

export DEBIAN_FRONTEND=noninteractive

apt install -y dialog locales
apt install -y systemd-sysv
apt install -y libterm-readline-gnu-perl

dpkg-divert --local --rename --add /sbin/initctl
ln -s /bin/true /sbin/initctl

apt -y upgrade

# 基础环境
apt install -y sudo ubuntu-standard
# 系统检测
apt install -y discover os-prober laptop-detect
# casper、grub相关
apt install -y casper lupin-casper
apt install -y grub-common grub-gfxpayload-lists grub-pc grub-pc-bin grub2-common

# 网络工具
apt install -y openssh-server network-manager wireless-tools
# 常用工具
apt install -y vim parted tmux iputils-ping iproute2 net-tools resolvconf
# 内核
apt install -y fdisk
apt install -y --no-install-recommends linux-generic
# 基础图形支持
apt install -y xserver-xorg xterm xinit
# 麒麟主题
apt install -y ubuntukylin-theme plymouth-themes
# 显示管理
apt install -y lightdm

## openbox
apt install -y hsetroot openbox
## ukui
apt install -y ukui-desktop-environment ukui-greeter # ukui-kwin

# 安装器：kylin-os-installer
apt install -y kylin-os-installer x11-xserver-utils udisks2

# dpkg-reconfigure locales
# dpkg-reconfigure resolvconf

cat <<EOF >/etc/NetworkManager/NetworkManager.conf
[main]
rc-manager=resolvconf
plugins=ifupdown,keyfile
dns=dnsmasq

[ifupdown]
managed=false
EOF

dpkg-reconfigure network-manager

# cleanup the chroot environment
truncate -s 0 /etc/machine-id
rm /sbin/initctl
dpkg-divert --rename --remove /sbin/initctl

apt purge -y packagekit
apt autoremove -y
# ==================== create system ====================

# ==================== config system ====================
# ssh允许root登录
if [ -f /etc/ssh/sshd_config ]; then
    if [ ! -f /etc/ssh/sshd_config.bak ]; then
        cp /etc/ssh/sshd_config{,.bak}
    fi

    if ! grep -q "^PermitRootLogin yes$" /etc/ssh/sshd_config; then
        sed -i '/PermitRootLogin/d' /etc/ssh/sshd_config
        echo 'PermitRootLogin yes' >>/etc/ssh/sshd_config
    fi
fi

# Network Manager
if [ -d /etc/NetworkManager/conf.d ]; then
    touch /etc/NetworkManager/conf.d/10-globally-managed-devices.conf
fi

if [ -f "/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf" ]; then
    # 使用openbox，登录界面允许root
    # sed -i '$a greeter-show-manual-login=true' /usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf
    cat >/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf <<EOF
[Seat:*]
user-session=openbox
greeter-show-manual-login=true
EOF
fi

# 设置默认登录用户
cat >/etc/lightdm/lightdm.conf <<EOF
[SeatDefaults]
autologin-guest=false
autologin-user=root
autologin-user-timeout=0
EOF

# 最终系统使用的源
cat >/etc/apt/sources.list <<EOF
deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main universe multiverse restricted
EOF

# hostname
echo kylinos >/etc/hostname

# root密码
echo root:123456 | chpasswd

# 清理
rm -rf /var/cache/apt/archives/*.deb
rm -rf /var/cache/apt/*.bin
rm -rf /var/lib/apt/lists/*_Packages
rm -rf /var/lib/apt/lists/*_InRelease
rm -rf /root/.bash_history
rm -rf /tmp/*
rm -rf ~/.bash_history
# ==================== config system ====================

export HISTSIZE=0
exit
