set -e

export HOME=/root
export LC_ALL=C

apt update
apt install -y systemd-sysv
apt install -y dialog locales

# 扩展根分区大小
apt install -y resize-assistant
# set background
apt install -y simple-wallpaper

# libopen62541-1 -> libmbedcrypto3 libmbedx509-0 libmbedtls12
apt install -y libopen62541-1

# 制作过程中使用的源
cat >/etc/apt/sources.list <<EOF
deb http://172.29.157.20/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
# kvisor, 协议包
deb http://172.29.157.20/my_mirror/kvisor v101-0621 main
EOF

apt update
export DEBIAN_FRONTEND=noninteractive

# 网络工具
apt install -y openssh-server network-manager netplan.io iputils-ping iproute2 net-tools
# 常用工具
apt install -y rsyslog vim sudo tmux mate-terminal
# 内核
apt install -y fdisk linux-generic
# 内核模块[可选]
# apt install -y linux-modules-5.4.18-35-generic
# 基础图形支持
apt install -y xserver-xorg xterm xinit
# 主题、字体相关
apt install -y ubuntukylin-theme plymouth-themes language-pack-zh-hans
# 显示管理
apt install -y lightdm
# openbox
apt install -y openbox hsetroot

# kysec包列表
apt install -y kysec-daemon kysec-sync-daemon kysec-utils libkysec libkysec-extend libkysecwhlist libpam-kysec security-switch kmod kyseclog-daemon libkysec-adv libkysec-core
# 安全相关补充
apt install -y kysec-auth auditd

# selinux包列表
apt install -y selinux-utils libselinux1 python3-selinux selinux-policy-ukmls selinux-policy-common libsemanage1 policycoreutils policycoreutils-python-utils python3-sepolgen python3-sepolicy secilc setools python3-setools

# alpha 版本集成
apt install -y qt5-default python2 python3

# kvisor
apt install -y kylin-hypervisor linux-image-5.4.18-60-generic linux-modules-5.4.18-60-generic linux-modules-extra-5.4.18-60-generic

# canopend 和 ethercat 放在内部源，其中ethercat 与 linux kernel 版本有关
apt install -y kylin-canopend kylin-ethercat

# 工业协议
apt install -y libmodbus5 mosquitto can-utils libcoap2-bin

# kvisor 设置 python3.9 为默认版本
update-alternatives --install /usr/bin/python python /usr/bin/python3.9 1

# overlayroot
apt install -y overlayroot

# ========== config ==========
# ssh允许root登录
if [ -f /etc/ssh/sshd_config ]; then
    if [ ! -f /etc/ssh/sshd_config.bak ]; then
        cp /etc/ssh/sshd_config{,.bak}
    fi

    if ! grep -q "^PermitRootLogin yes$" /etc/ssh/sshd_config; then
        sed -i '/PermitRootLogin/d' /etc/ssh/sshd_config
        echo 'PermitRootLogin yes' >>/etc/ssh/sshd_config
    fi
fi

# Network Manager
cat <<EOF >/etc/NetworkManager/NetworkManager.conf
[main]
rc-manager=resolvconf
plugins=ifupdown,keyfile
dns=dnsmasq

[ifupdown]
managed=false
EOF

dpkg-reconfigure network-manager

if [ -d /etc/NetworkManager/conf.d ]; then
    touch /etc/NetworkManager/conf.d/10-globally-managed-devices.conf
fi

# 设置默认语言为中文
echo "zh_CN.UTF-8 UTF-8
en_US.UTF-8 UTF-8" >/etc/locale.gen
echo 'LANG=zh_CN.UTF-8' >/etc/default/locale
echo 'LANGUAGE="zh_CN:zh"' >>/etc/default/locale
locale-gen

# 使用openbox，登录界面允许root
cat >/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf <<EOF
[Seat:*]
user-session=openbox
greeter-show-manual-login=true
EOF

# 设置默认登录用户
cat >/etc/lightdm/lightdm.conf <<EOF
[SeatDefaults]
autologin-guest=false
autologin-user=root
autologin-user-timeout=0
EOF

# 最终系统使用的源
cat >/etc/apt/sources.list <<EOF
deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main universe multiverse restricted
EOF

# 设置 hostname
echo kylinos >/etc/hostname

# 设置 root密码
echo root:qwer1234 | chpasswd

# 添加 kylin 用户
if ! awk -F: '{print $1}' /etc/passwd | grep -q ekylin; then
    USERNAME="ekylin"
    PASSWORD="qwer1234"
    useradd -m -s /bin/bash -u 2000 "${USERNAME}"
    usermod -c "${USERNAME}" "${USERNAME}"
    echo "${USERNAME}:${PASSWORD}" | chpasswd
    adduser ${USERNAME} sudo
    adduser ${USERNAME} adm
fi

# 关闭kysec
if [ -f "/etc/kylin-security.conf" ]; then
    sed -i "/kysec_status/ {s/2/0/}" /etc/kylin-security.conf
fi

# 设置默认终端
if [ -x "/usr/bin/mate-terminal.wrapper" ]; then
    update-alternatives --set x-terminal-emulator /usr/bin/mate-terminal.wrapper
fi

# 开机执行resize
if dpkg -l | grep ^ii | awk '{print $2}' | grep -q "^resize-assistant$"; then
    systemctl enable resize-assistant
fi

# 清理
rm -rf /var/cache/apt/archives/*.deb
rm -rf /var/cache/apt/*.bin
rm -rf /var/lib/apt/lists/*_Packages
rm -rf /var/lib/apt/lists/*_InRelease
rm -rf /root/.bash_history
rm -rf /tmp/*
rm -rf ~/.bash_history
# ========== config ==========

export HISTSIZE=0
exit
