#!/bin/bash
set -e

export HOME=/root
export LC_ALL=C

apt update
apt install -y systemd-sysv
apt install -y dialog locales

# 扩展根分区大小
apt install -y resize-assistant
# set background
apt install -y simple-wallpaper

# 制作过程中使用的源
cat >/etc/apt/sources.list <<EOF
deb http://172.29.157.20/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
EOF

apt update
export DEBIAN_FRONTEND=noninteractive

# 网络工具
apt install -y openssh-server network-manager netplan.io iputils-ping iproute2 net-tools
# 常用工具
apt install -y rsyslog vim sudo tmux mate-terminal file cpio lsof fdisk parted sudo bash-completion gnome-terminal
# 常用开发工具
# apt install -y make cmake gcc g++
# 内核
apt install -y fdisk linux-generic
# 内核模块[可选]
# apt install -y linux-modules-5.4.18-35-generic
# 基础图形支持
apt install -y xserver-xorg xterm xinit
# 主题、字体相关
apt install -y ubuntukylin-theme plymouth-themes language-pack-zh-hans
# 显示管理
apt install -y lightdm
# openbox
apt install -y openbox hsetroot

# 麒麟影音 ffmpeg libopengl [不影响运行，暂时注掉，加快制作速度]
# apt install -y kylin-video ffmpeg libopengl-dev libopengl-perl libopengl-image-perl

# 卸载一些无用的包
apt purge -y unity-settings-daemon || true
apt purge -y gnome-control-center || true
apt purge -y packagekit || true
apt autoremove -y

# ==================== config system ====================
# ssh允许root登录
if [ -f /etc/ssh/sshd_config ]; then
    if [ ! -f /etc/ssh/sshd_config.bak ]; then
        cp /etc/ssh/sshd_config{,.bak}
    fi

    if ! grep -q "^PermitRootLogin yes$" /etc/ssh/sshd_config; then
        sed -i '/PermitRootLogin/d' /etc/ssh/sshd_config
        echo 'PermitRootLogin yes' >>/etc/ssh/sshd_config
    fi
fi

# Network Manager
cat <<EOF >/etc/NetworkManager/NetworkManager.conf
[main]
rc-manager=resolvconf
plugins=ifupdown,keyfile
dns=dnsmasq

[ifupdown]
managed=false
EOF

dpkg-reconfigure network-manager

if [ -d /etc/NetworkManager/conf.d ]; then
    touch /etc/NetworkManager/conf.d/10-globally-managed-devices.conf
fi

# 设置默认语言为中文
echo "zh_CN.UTF-8 UTF-8
en_US.UTF-8 UTF-8" >/etc/locale.gen
echo 'LANG=zh_CN.UTF-8' >/etc/default/locale
echo 'LANGUAGE="zh_CN:zh"' >>/etc/default/locale
locale-gen

# 使用openbox，登录界面允许root
cat >/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf <<EOF
[Seat:*]
user-session=openbox
greeter-show-manual-login=true
EOF

# 设置默认登录用户
cat >/etc/lightdm/lightdm.conf <<EOF
[SeatDefaults]
autologin-guest=false
autologin-user=root
autologin-user-timeout=0
EOF

# 最终系统使用的源
cat >/etc/apt/sources.list <<EOF
deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main universe multiverse restricted
EOF

# 设置 hostname
echo kylinos >/etc/hostname

# 设置 root密码
echo root:qwer1234 | chpasswd

# 添加 kylin 用户
if ! awk -F: '{print $1}' /etc/passwd | grep -q ekylin; then
    USERNAME="ekylin"
    PASSWORD="qwer1234"
    useradd -m -s /bin/bash -u 2000 "${USERNAME}"
    usermod -c "${USERNAME}" "${USERNAME}"
    echo "${USERNAME}:${PASSWORD}" | chpasswd
    adduser ${USERNAME} sudo
    adduser ${USERNAME} adm
fi

# 关闭kysec
if [ -f "/etc/kylin-security.conf" ]; then
    sed -i "/kysec_status/ {s/2/0/}" /etc/kylin-security.conf
fi

# 设置默认终端
if [ -x "/usr/bin/mate-terminal.wrapper" ]; then
    update-alternatives --set x-terminal-emulator /usr/bin/mate-terminal.wrapper
fi

# 开机执行resize
if dpkg -l | grep ^ii | awk '{print $2}' | grep -q "^resize-assistant$"; then
    systemctl enable resize-assistant
fi

# 清理
rm -rf /var/cache/apt/archives/*.deb
rm -rf /var/cache/apt/*.bin
rm -rf /var/lib/apt/lists/*_Packages
rm -rf /var/lib/apt/lists/*_InRelease
rm -rf /root/.bash_history
rm -rf /tmp/*
rm -rf ~/.bash_history
# ==================== config system ====================

export HISTSIZE=0
exit
