#!/bin/bash
set -e

export HOME=/root
export LC_ALL=C

apt update

# resize
apt install -y resize-assistant

# ==================== create system ====================
# 制作过程中使用的源
cat >/etc/apt/sources.list <<EOF
deb http://172.29.220.247:8000 10.0 main restricted universe multiverse
EOF

apt update

export DEBIAN_FRONTEND=noninteractive

apt install -y dialog locales
apt install -y systemd-sysv

# 网络工具
apt install -y openssh-server network-manager iputils-ping iproute2 net-tools
# 常用工具 fdisk -> util-linux
apt install -y vim file cpio lsof parted sudo bash-completion gnome-terminal
# 常用开发工具
apt install -y make cmake gcc g++
# 内核模块[可选]
# apt install -y linux-image-generic
# 基础图形支持
apt install -y xserver-xorg xterm xinit
# 麒麟主题
# apt install -y ubuntukylin-theme
apt install -y plymouth-themes
# 显示管理
apt install -y lightdm

# v10 上没有 ukui-desktop-environment，改为 mate-desktop-environment
apt install -y mate-desktop mate-desktop-environment ukui-greeter
# 进入桌面提示有两个提示，组件不全。默认没有启动窗口管理器，手动拷贝过去
# cp -f /usr/share/applications/marco.desktop /etc/xdg/autostart/
# 装了 kylin-menu mate-indicators 之后，没有组件提示问题了，也不用拷贝marco.desktop了
apt install -y apt-utils kylin-settings kylin-menu mate-indicators
# 音频播放，插着HDMI和耳机，需要打开设置，选择内置声音输出设备
apt install -y kylin-control-center ukui-screensaver

cat <<EOF >/etc/NetworkManager/NetworkManager.conf
[main]
rc-manager=resolvconf
plugins=ifupdown,keyfile
dns=dnsmasq

[ifupdown]
managed=false
EOF

dpkg-reconfigure network-manager

# 中文
apt install -y language-pack-zh-hans
echo "zh_CN.UTF-8 UTF-8
en_US.UTF-8 UTF-8" >/etc/locale.gen
echo 'LANG=zh_CN.UTF-8' >/etc/default/locale
echo 'LANGUAGE="zh_CN:zh"' >>/etc/default/locale
locale-gen

# 麒麟影音 ffmpeg libopengl
# 没有 libopengl-dev
apt install -y kylin-video ffmpeg libopengl-perl libopengl-image-perl

# 卸载一些无用的包【比上个版本多的操作，不能切换用户】
# apt purge -y unity-settings-daemon
# apt purge -y gnome-control-center

apt purge -y packagekit || true
apt autoremove -y
# ==================== create system ====================

# ==================== config system ====================
# ssh允许root登录
if [ -f /etc/ssh/sshd_config ]; then
    if [ ! -f /etc/ssh/sshd_config.bak ]; then
        cp /etc/ssh/sshd_config{,.bak}
    fi

    if ! grep -q "^PermitRootLogin yes$" /etc/ssh/sshd_config; then
        sed -i '/PermitRootLogin/d' /etc/ssh/sshd_config
        echo 'PermitRootLogin yes' >>/etc/ssh/sshd_config
    fi
fi

# Network Manager
if [ -d /etc/NetworkManager/conf.d ]; then
    touch /etc/NetworkManager/conf.d/10-globally-managed-devices.conf
fi

if [ -f "/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf" ]; then
    # 登录界面允许root
    sed -i '$a greeter-show-manual-login=true' /usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf
fi

# 设置默认登录用户
cat >/etc/lightdm/lightdm.conf <<EOF
[SeatDefaults]
autologin-guest=false
# autologin-user=root
autologin-user-timeout=0
EOF

# 最终系统使用的源
cat >/etc/apt/sources.list <<EOF
deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.0 main universe multiverse restricted
EOF

# hostname
echo kylinos >/etc/hostname

# root密码
echo root:123456 | chpasswd

# 添加 kylin 用户
USERNAME="kylin"
PASSWORD="qwer1234"
useradd -m -s /bin/bash -u 2000 "${USERNAME}"
usermod -c "${USERNAME}" "${USERNAME}"
echo "${USERNAME}:${PASSWORD}" | chpasswd
adduser ${USERNAME} sudo
adduser ${USERNAME} adm

# 清理
rm -rf /var/cache/apt/archives/*.deb
rm -rf /var/cache/apt/*.bin
rm -rf /var/lib/apt/lists/*_Packages
rm -rf /var/lib/apt/lists/*_InRelease
rm -rf /root/.bash_history
rm -rf /tmp/*
rm -rf ~/.bash_history
# ==================== config system ====================

export HISTSIZE=0
exit
