#!/bin/bash

ROOTFS_DIR="squashfs"

CURRENT_DIR=$(dirname $(readlink -f $0))

function do_config() {

    # 宿主机挂载 设备节点 到 rootfs
    sudo mount --bind /dev ${ROOTFS_DIR}/dev
    sudo mount --bind /run ${ROOTFS_DIR}/run
    sudo mount -t devpts devpts ${ROOTFS_DIR}/dev/pts
    sudo mount -t proc proc ${ROOTFS_DIR}/proc
    sudo mount -t sysfs sysfs ${ROOTFS_DIR}/sys

    sudo chroot ${ROOTFS_DIR}
    export HOME=/root
    export LC_ALL=C

    apt purge -y firefox firefox-locale-zh-hans
    apt purge -y wps-office sogouimebs nvidia-graphics-driver browser360-cn-stable qaxbrowser-safe-stable sc-reader pantum
    apt purge -y ubiquity ubiquity-casper ubiquity-frontend-gtk ubiquity-slideshow-kylin ubiquity-ubuntu-artwork

    # 最终系统使用的源
    cat >/etc/apt/sources.list <<EOF
deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.0 main universe multiverse restricted
EOF

    rm /etc/apt/sources.list.d/*

    apt update
    apt install -y kylin-video ffmpeg libopengl-perl libopengl-image-perl

    if [ -f "/usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf" ]; then
        # 登录界面允许root
        sed -i '$a greeter-show-manual-login=true' /usr/share/lightdm/lightdm.conf.d/95-ukui-greeter.conf
    fi

    # hostname
    echo kylinos >/etc/hostname

    # root密码
    echo root:qwer1234 | chpasswd

    # 添加 kylin 用户
    USERNAME="kylin"
    PASSWORD="qwer1234"
    useradd -m -s /bin/bash -u 2000 "${USERNAME}"
    usermod -c "${USERNAME}" "${USERNAME}"
    echo "${USERNAME}:${PASSWORD}" | chpasswd
    adduser ${USERNAME} sudo
    adduser ${USERNAME} adm

    # 清理
    rm -rf /var/cache/apt/archives/*.deb
    rm -rf /var/cache/apt/*.bin
    rm -rf /var/lib/apt/lists/*_Packages
    rm -rf /var/lib/apt/lists/*_InRelease
    rm -rf /root/.bash_history
    rm -rf /tmp/*
    rm -rf ~/.bash_history

    export HISTSIZE=0
    exit

    # 卸载宿主设备节点
    sudo umount ${ROOTFS_DIR}/dev/pts
    sudo umount ${ROOTFS_DIR}/dev
    sudo umount ${ROOTFS_DIR}/run
    sudo umount ${ROOTFS_DIR}/sys
    sudo umount ${ROOTFS_DIR}/proc
}

function do_fstab() {
    sudo cp -v ${CURRENT_DIR}/etc/fstab ${ROOTFS_DIR}/etc
}

function do_ext4() {
    size=6656 # 指定 6.5G 大小

    dd if=/dev/zero of=${ROOTFS_DIR}.ext4 bs=1M count=${size} status=progress
    sudo mkdir -p ./ext4_mount
    yes | mkfs.ext4 ${ROOTFS_DIR}.ext4
    sudo mount ${ROOTFS_DIR}.ext4 ./ext4_mount
    sudo cp -rf -a ${ROOTFS_DIR}/* ./ext4_mount
    sudo umount ./ext4_mount
    sudo rmdir ./ext4_mount
    echo "tar zcvf ${ROOTFS_DIR}.ext4.tar.gz ${ROOTFS_DIR}.ext4"
    tar zcf ${ROOTFS_DIR}.ext4.tar.gz ${ROOTFS_DIR}.ext4
    md5sum ${ROOTFS_DIR}.ext4.tar.gz | awk '{print $1}' >${ROOTFS_DIR}.ext4.tar.gz.md5
}

do_$1
