############################################################### mylog
MY_LOG_FILE=/var/tmp/my_debootstrap_test.log
: >${MY_LOG_FILE}

mylog() {
    if [ $# -eq 1 ]; then
        echo "---> $1" >>${MY_LOG_FILE}
    else
        filename=$1
        shift
        echo "$@" >/var/tmp/${filename}
    fi
}

mylog2() {
    echo "---> $@" >>${MY_LOG_FILE}
}

smallyes() {
    YES="${1-y}"
    while echo "$YES" 2>/dev/null; do :; done
}

########## functions for first stage ##########
conditional_cp() {
    if [ ! -e "$2/$1" ]; then
        if [ -L "$1" ] && [ -e "$1" ]; then
            cat "$1" >"$2/$1"
        elif [ -e "$1" ]; then
            cp "$1" "$2/$1"
        fi
    fi
}

setup_merged_usr() {
    link_dir="bin sbin lib lib32 lib64 libx32"
    local dir
    for dir in $link_dir; do
        mkdir -p "$TARGET/usr/$dir"
        ln -s usr/"$dir" "$TARGET/$dir"
    done
}

extract_dpkg_deb_data() {
    DEB_LIST1=/data/temp/01-kylin-rootfs-first-and-second/1.txt
    # DEB_DIR=/data/temp/kylin-rootfs
    if [ -f "${DEB_LIST1}" ]; then
        for i in $(cat ${DEB_LIST1}); do
            # i="${DEB_DIR}${i}"
            i="$(dirname ${DEB_LIST1})${i}"
            echo $i
            dpkg-deb --fsys-tarfile $i | tar -k -xf -
        done
    else
        echo "${DEB_LIST1} is not exist!!"
    fi
}

# Create a device node if it does not exist. By default, the mode is 666.
mknod_if_needed() {
    local device type major minor mode
    device="$1"
    type="$2"
    major="$3"
    minor="$4"
    mode="${5:-666}"

    if [ ! -e "$device" ]; then
        mknod -m "$mode" "$device" "$type" "$major" "$minor"
    fi
}

setup_devices_simple() {
    # The list of devices that can be created in a container comes from
    # src/core/cgroup.c in the systemd source tree.
    mknod_if_needed "$TARGET/dev/null" c 1 3
    mknod_if_needed "$TARGET/dev/zero" c 1 5
    mknod_if_needed "$TARGET/dev/full" c 1 7
    mknod_if_needed "$TARGET/dev/random" c 1 8
    mknod_if_needed "$TARGET/dev/urandom" c 1 9
    mknod_if_needed "$TARGET/dev/tty" c 5 0
    if [ ! "$CONTAINER" = "systemd-nspawn" ]; then
        mknod_if_needed "$TARGET/dev/console" c 5 1
    fi
    # To avoid pre-exist directory causes error, specify "-p" option
    mkdir -p "$TARGET/dev/pts/" "$TARGET/dev/shm/"
    # Inside a container, we might not be allowed to create /dev/ptmx.
    # If not, do the next best thing.
    if ! mknod_if_needed "$TARGET/dev/ptmx" c 5 2; then
        warning MKNOD "Could not create /dev/ptmx, falling back to symlink. This chroot will require /dev/pts mounted with ptmxmode=666"
        ln -sf pts/ptmx "$TARGET/dev/ptmx"
    fi
    ln -sf /proc/self/fd "$TARGET/dev/fd"
    ln -sf /proc/self/fd/0 "$TARGET/dev/stdin"
    ln -sf /proc/self/fd/1 "$TARGET/dev/stdout"
    ln -sf /proc/self/fd/2 "$TARGET/dev/stderr"
}
########## functions for first stage ##########

first_stage_setup() {
    TARGET=$(pwd)
    echo "TARGET=${TARGET}"

    setup_merged_usr

    extract_dpkg_deb_data

    mkdir -p "$TARGET/var/lib/dpkg"
    : >"$TARGET/var/lib/dpkg/status"
    : >"$TARGET/var/lib/dpkg/available"

    mkdir -p "$TARGET/etc"
    conditional_cp /etc/resolv.conf "$TARGET"
    conditional_cp /etc/hostname "$TARGET"

    if [ ! -e "$TARGET/etc/fstab" ]; then
        echo '# UNCONFIGURED FSTAB FOR BASE SYSTEM' >"$TARGET/etc/fstab"
        chown 0:0 "$TARGET/etc/fstab"
        chmod 644 "$TARGET/etc/fstab"
    fi

    setup_devices_simple

    echo "deb http://172.29.157.20/kylin v101 main restricted universe multiverse" >"$TARGET/etc/apt/sources.list"
}

########## functions for second stage ##########

in_target_failmsg() {
    local code msg arg
    code="$1"
    msg="$2"
    arg="$3"
    shift
    shift
    shift
    if ! PATH=/sbin:/usr/sbin:/bin:/usr/bin eval "$CHROOT_CMD \"\$@\""; then
        echo "$code" "$msg" "$arg" >>"$TARGET/debootstrap/debootstrap.log"
        # Try to point user at actual failing package.
        msg="See %s for details"
        if [ -e "$TARGET/debootstrap/debootstrap.log" ]; then
            arg="$TARGET/debootstrap/debootstrap.log"
            local pkg="$(grep '^dpkg: error processing ' "$TARGET/debootstrap/debootstrap.log" | head -n 1 | sed 's/\(error processing \)\(package \|archive \)/\1/' | cut -d ' ' -f 4)"
            if [ -n "$pkg" ]; then
                msg="$msg (possibly the package $pkg is at fault)"
            fi
        else
            arg="the log"
        fi
        echo "$code" "$msg" "$arg" >>"$TARGET/debootstrap/debootstrap.log"
        return 1
    fi
    return 0
}

in_target() {
    in_target_failmsg IN_TARGET_FAIL "Failure trying to run: %s" "$CHROOT_CMD $*" "$@"
}

debfor() {
    (
        while read pkg path; do
            for p in "$@"; do
                [ "$p" = "$pkg" ] || continue
                echo "$path"
                mylog "====== $path"
            done
        done <"$TARGET/debootstrap/debpaths"
    )
}

x_feign_install() {
    local pkg="$1"
    local deb="$(debfor $pkg)"
    mylog "deb=${deb}"
    local ver="$(in_target dpkg-deb -f "$deb" Version)"
    mylog "ver=${ver}"

    mkdir -p "$TARGET/var/lib/dpkg/info"

    echo "Package: $pkg
Version: $ver
Maintainer: unknown
Status: install ok installed" >>"$TARGET/var/lib/dpkg/status"

    touch "$TARGET/var/lib/dpkg/info/${pkg}.list"
}

x_core_install() {
    smallyes '' | in_target dpkg --force-depends --install $(debfor "$@")
}

setup_dselect_method() {
    case "$1" in
    apt)
        mkdir -p "$TARGET/var/lib/dpkg"
        echo "apt apt" >"$TARGET/var/lib/dpkg/cmethopt"
        chmod 644 "$TARGET/var/lib/dpkg/cmethopt"
        ;;
    *)
        error 1 UNKNOWNDSELECT "unknown dselect method"
        ;;
    esac
}

setup_available() {
    for p in "$@"; do
        sed -n "/^Package: ${p}$/,/^$/p" /var/lib/apt/lists/*_Packages
    done >"$TARGET/var/lib/dpkg/available"
}

########## functions for second stage ##########

second_stage() {
    TARGET="/"

    x_feign_install dpkg

    DEBIAN_FRONTEND=noninteractive
    DEBCONF_NONINTERACTIVE_SEEN=true
    export DEBIAN_FRONTEND DEBCONF_NONINTERACTIVE_SEEN

    ln -sf mawk "$TARGET/usr/bin/awk"
    x_core_install base-passwd
    x_core_install base-files
    x_core_install dpkg

    if [ ! -e "$TARGET/etc/localtime" ]; then
        ln -sf /usr/share/zoneinfo/UTC "$TARGET/etc/localtime"
    fi

    LIBC="libc6"
    x_core_install $LIBC

    x_core_install perl-base

    rm "$TARGET/usr/bin/awk"
    x_core_install mawk

    x_core_install debconf

    DEBOOTSTRAP_DIR=/debootstrap
    required=$(cat "$DEBOOTSTRAP_DIR/required")
    base=$(cat "$DEBOOTSTRAP_DIR/base")
    all_debs="$required $base"

    exec 7>&1
    dpkg --status-fd 8 --force-depends --unpack $(debfor $required) 8>&1 1>&7 || echo EXITCODE $?

    # echo "#!/bin/sh"
    echo "exit 101" >"$TARGET/usr/sbin/policy-rc.d"
    chmod 755 "$TARGET/usr/sbin/policy-rc.d"

    mv "$TARGET/sbin/start-stop-daemon" "$TARGET/sbin/start-stop-daemon.REAL"

    echo "Warning: Fake start-stop-daemon called, doing nothing" >"$TARGET/sbin/start-stop-daemon"
    chmod 755 "$TARGET/sbin/start-stop-daemon"

    setup_dselect_method apt

    dpkg --status-fd 8 --configure --pending --force-configure-any --force-depends 8>&1 1>&7 || echo EXITCODE $?

    setup_available $required $base
}

########## pkgdetails

pkgdetails_field() {
    # uniq field mirror Packages values...
    perl -le '
$unique = shift @ARGV; $field = lc(shift @ARGV); $mirror = shift @ARGV;
%fields = map { $_, 0 } @ARGV;
$prevpkg = "";
$chksumfield = lc($ENV{DEBOOTSTRAP_CHECKSUM_FIELD}).":";
while (<STDIN>) {
	if (/^([^:]*:)\s*(.*)$/) {
		$f = lc($1); $v = $2;
		if ($f eq "package:") {
			$last = 0;
			$pkg = $v;
			if ($pkg ne $prevpkg) {
				print $output if defined $output;
				if ($unique && defined $output_val) {
					delete $fields{$output_val};
					$last = 1 unless keys %fields;
				}
				$prevpkg = $pkg;
			}
			undef $output;
			undef $output_val;
			last if $last;
		}
		$ver = $v if ($f eq "version:");
		$arc = $v if ($f eq "architecture:");
		$fil = $v if ($f eq "filename:");
		$chk = $v if ($f eq $chksumfield);
		$siz = $v if ($f eq "size:");
		$val = $v if ($f eq $field);
	} elsif (/^$/) {
		if (defined $val && defined $fields{$val}) {
			$output = sprintf "%s %s %s %s %s %s %s",
			 $pkg, $ver, $arc, $mirror, $fil, $chk, $siz;
			$output_val = $val;
		}
		undef $val;
	}
}
print $output if defined $output;
delete $fields{$output_val} if $unique && defined $output_val;
for $v (keys %fields) {
	printf ("%s -\n", $v) if ($unique);
}
' "$@"
}

pkgdetails_perl() {
    # mylog2 "$@"
    if [ "$1" = "WGET%" ]; then
        shift
        perl -e '
$v = 0;
$allow_percentage = 0;
while (read STDIN, $x, 1) {
	if ($x =~ m/\s/) {
		$allow_percentage = 1;
	} elsif ($allow_percentage and $x =~ m/\d/) {
		$v *= 10;
		$v += $x;
	} elsif ($allow_percentage and $x eq "%") {
		printf "P: %d %d%s\n", int($v / 100.0 * ($ARGV[1] - $ARGV[0]) + $ARGV[0]), $ARGV[2], ($#ARGV == 3 ? " $ARGV[3]" : "");
		$v = 0;
	} else {
		$v = 0;
		$allow_percentage = 0;
	}
}' "$@"
    elif [ "$1" = "GETDEPS" ]; then
        local pkgdest="$2"
        shift
        shift
        LC_ALL=C grep "$gropt" '^$|^Package:|^Depends:|^Pre-Depends:' $pkgdest | perl -e '
%seen = map { $_ => 1 } @ARGV;
while (<STDIN>) {
	if (/^Package: (.*)$/) {
		$pkg = $1;
		next;
	} elsif (/^$/) {
                $in = 0;
                next;
        }
	$in = 1 if $seen{$pkg};
	if ($in and (/^Depends: (.*)$/ or /^Pre-Depends: (.*)$/)) {
		for $d (split /\s*,\s*/, $1) {
			$d =~ s/\s*[|].*$//;
			$d =~ s/\s*[(].*[)]\s*//;
			$d =~ s/:.*//;
			$depends{$d} = 1;
		}
	}
}
	foreach (sort keys %depends) {
	  print "$_\n";
	}
' "$@"
    elif [ "$1" = "PKGS" ]; then
        local m="$2"
        local p="$3"
        shift
        shift
        shift
        LC_ALL=C grep "$gropt" '^$|^Architecture:|^Filename:|^MD5sum:|^Package:|^Priority:|^SHA256:|^Size:|^Version:|^Depends:|^Pre-Depends:' "$p" | pkgdetails_field 1 Package: "$m" "$@"
    elif [ "$1" = "FIELD" ]; then
        local f="$2"
        local m="$3"
        local p="$4"
        shift
        shift
        shift
        shift
        LC_ALL=C grep "$gropt" '^$|^Package:|^Priority:' "$p" | pkgdetails_field 0 "$f" "$m" "$@"
    elif [ "$1" = "STANZAS" ]; then
        local pkgdest="$2"
        shift
        shift
        perl -e '
my $accum = "";
%seen = map { $_ => 1 } @ARGV;
while (<STDIN>) {
	$accum .= $_;
	$in = 1 if (/^Package: (.*)$/ && $seen{$1});
	if ($in and /^$/) {
		print $accum;
		if (substr($accum, -1) != "\n") {
			print "\n\n";
		} elsif (substr($accum, -2, 1) != "\n") {
			print "\n";
		}
		$in = 0;
	}
	$accum = "" if /^$/;
}' "$@" <"$pkgdest"
    fi
}

get_debs() {
    local field m1 c path pkgdest
    field="$1"
    shift
    for m1 in $MIRRORS; do
        for s in $SUITE $EXTRA_SUITES; do
            for c in $COMPONENTS; do
                path="dists/$s/$c/binary-$ARCH/Packages"
                # pkgdest="$TARGET/$($DLDEST pkg "$s" "$c" "$ARCH" "$m1" "$path")"
                # pkgdest="/data/git/gogs/kylin/172.29.157.20/my_mirror/kylin/dists/v101/main/binary-amd64/Packages"
                pkgdest="${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists/v101/main/binary-amd64/Packages"
                echo "$(pkgdetails_perl FIELD "$field" "$m1" "$pkgdest" "$@" | sed 's/ .*//')"
            done
        done
    done
}

without() {
    # usage:  without "a b c" "a d" -> "b" "c"
    (
        echo "$1" | tr ' ' '\n' | sort | uniq
        echo "$2" "$2" | tr ' ' '\n'
    ) | sort | uniq -u | tr '\n' ' '
    echo
}

resolve_deps() {
    local m1="${MIRRORS%% *}"

    local PKGS="$*"
    local ALLPKGS="$PKGS"
    local ALLPKGS2=""
    while [ "$PKGS" != "" ]; do
        local NEWPKGS=""
        for s in $SUITE $EXTRA_SUITES; do
            for c in ${COMPONENTS:-$(echo ${USE_COMPONENTS} | tr '|' ' ')}; do
                local path="dists/$s/$c/binary-$ARCH/Packages"
                # local pkgdest="$TARGET/$($DLDEST pkg "$s" "$c" "$ARCH" "$m1" "$path")"
                local pkgdest="${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists/v101/main/binary-amd64/Packages"
                NEWPKGS="$NEWPKGS $(pkgdetails_perl GETDEPS "$pkgdest" $PKGS)"
            done
        done
        mylog "newpkgs.txt" "$(echo ${NEWPKGS} | xargs | tr ' ' '\n')"
        PKGS=$(echo "$PKGS $NEWPKGS" | tr ' ' '\n' | sort | uniq)
        local REALPKGS=""
        for s in $SUITE $EXTRA_SUITES; do
            for c in ${COMPONENTS:-$(echo ${USE_COMPONENTS} | tr '|' ' ')}; do
                local path="dists/$s/$c/binary-$ARCH/Packages"
                # local pkgdest="$TARGET/$($DLDEST pkg "$s" "$c" "$ARCH" "$m1" "$path")"
                local pkgdest="${TARGET_DIR}/172.29.157.20/my_mirror/kylin/dists/v101/main/binary-amd64/Packages"
                REALPKGS="$REALPKGS $(pkgdetails_perl PKGS REAL "$pkgdest" $PKGS | sed -n 's/ .*REAL.*$//p')"
            done
        done
        mylog "realpkgs.txt" "$(echo ${REALPKGS} | xargs | tr ' ' '\n')"
        PKGS="$REALPKGS"
        ALLPKGS2=$(echo "$PKGS $ALLPKGS" | tr ' ' '\n' | sort | uniq)
        PKGS=$(without "$ALLPKGS2" "$ALLPKGS")
        ALLPKGS="$ALLPKGS2"
    done
    echo "$ALLPKGS"
}

MIRRORS='http://172.29.157.20/my_mirror/kylin'
SUITE='v101'
COMPONENTS='main'
ARCH='amd64'
gropt="--perl-regexp"
TARGET_DIR="/var/tmp/indices"
if [ ! -d "${TARGET_DIR}" ]; then
    mkdir -p "${TARGET_DIR}"
fi

required="$(get_debs Priority: required)"
mylog "01-required.txt1" "${required}"
requiredX=$(echo "$required" | tr ' ' '\n' | sort | uniq)
required=$(resolve_deps "$requiredX")
mylog "01-required@deps.txt1" "$(echo ${required} | xargs | tr ' ' '\n')"

base="$(get_debs Priority: important)"
mylog "02-important.txt1" "${base}"
baseX=$(echo "$base" | tr ' ' '\n' | sort | uniq)
base=$(resolve_deps "$baseX")
mylog "02-important@deps.txt1" "$(echo ${base} | xargs | tr ' ' '\n')"
base=$(without "$base" "$required")
mylog "02-important@deps@without@required.txt1" "$(echo ${base} | xargs | tr ' ' '\n')"

# baseN=$(without "$baseX" "$requiredX")
# baseU=$(without "$baseX" "$baseN")
# echo ${baseU}
requiredX=$(without "$required" "$requiredX")
mylog "01-requiredX.txt1" "$(echo ${requiredX} | xargs | tr ' ' '\n')"
baseX=$(without "$base" "$baseX")
mylog "02-importantX.txt1" "$(echo ${baseX} | xargs | tr ' ' '\n')"
# echo "${requiredX}" | tr ' ' '\n' | wc -l
# echo ${baseX} | tr ' ' '\n' | wc -l

all_debs="$required $base"
echo "${all_debs}"
