#!/bin/bash

# set -e

# deb http://172.29.157.20/my_mirror/kylin v101 main
: 本地源操作 <<BLOCK
source /data/git/gogs/kylin/debootstrap/reprepro.sh

# 创建源
repo_create kylin v101
repo_create juniper v100

# 添加包
repo_add kylin v101 amd64 /data/KYLINALL/kylin-amd64/
repo_add kylin v101 arm64 /data/KYLINALL/kylin-arm64/
repo_add juniper v100 arm64 /data/KYLINALL/juniper-arm64/

repo_add kylin v101 amd64 debs/02-kylin-apt-keyring/kylin-apt-keyring_2022.04.12_all.deb

# 查看包列表
repo_list kylin v101 amd64
repo_list juniper v100 arm64

# 删除包
repo_remove kylin v101 amd64 all
repo_remove kylin v101 amd64 bash
BLOCK

# deb http://172.29.157.20/my_mirror/rk3568 v101 main
: rk3568 <<BLOCK
repo_create rk3568 v101

# 批量添加
for i in `find /data/my_mirror/rk3568-debs -name "*.deb"`
do
    repo_add rk3568 v101 arm64 $i
done

repo_list rk3568 v101 arm64
BLOCK

# deb http://172.29.157.20/my_mirror/kvisor v101-0621 main
: kylin-hypervisor <<BLOCK
repo_create kvisor v101-0621
repo_add kvisor v101-0621 arm64 /data/KYLINALL/kvisor-deb/kylin-hypervisor_1.0_arm64.deb
repo_add kvisor v101-0621 arm64 /data/KYLINALL/kvisor-deb/deb_package
repo_list kvisor v101-0621 arm64
BLOCK

# deb http://172.29.157.20/my_mirror/extra v101-0721 main
: 第三方其他包 <<BLOCK
repo_create extra v101-0721

repo_add extra v101-0721 arm64 /data/my_mirror/extra-debs/kylin-snapshottools_1.8.6.3.2-30_arm64.deb
repo_list extra v101-0721 arm64

repo_add extra v101-0721 amd64 /data/my_mirror/extra-debs/kylin-snapshottools_1.8.6.3.2-30_amd64.deb
repo_list extra v101-0721 amd64
BLOCK

# 镜像路径前缀
MIRROR_PREFIX="/data/my_mirror"

# 创建源
function repo_create() {
    TARGET=$1
    VERSION=$2

    # 源发布目录
    MIRROR_DIR="${MIRROR_PREFIX}/${TARGET}"
    # 创建配置
    mkdir -p ${MIRROR_DIR}/conf

    GPG_ID=$(gpg --list-keys yangzhen | sed -n '2p' | sed 's/ //g')

    # 导出 GPG 公钥
    if [ ! -f "${MIRROR_DIR}/public.key" ]; then
        gpg --armor -o ${MIRROR_DIR}/public.key --export ${GPG_ID}
    fi

    # create conf/distributions
    cat >${MIRROR_DIR}/conf/distributions <<EOF
Origin: origin-kylin
Label: label-kylin
Suite: stable
Codename: ${VERSION}
Version: 2022
Architectures: amd64 i386 arm64
Components: main
Description: Kylin Package Archives
SignWith: ${GPG_ID}
EOF

    # build tree
    reprepro --ask-passphrase -Vb ${MIRROR_DIR} export
}

# 添加包
function repo_add() {
    TARGET=$1
    VERSION=$2
    ARCH=$3
    DEBS_TARGET=$4

    # 源发布目录
    MIRROR_DIR="${MIRROR_PREFIX}/${TARGET}"

    if [ -d "${DEBS_TARGET}" ]; then
        # 批量添加
        reprepro --ask-passphrase -Vb ${MIRROR_DIR} -C main -A ${ARCH} includedeb ${VERSION} ${DEBS_TARGET}/*.deb
    elif [ -f "${DEBS_TARGET}" ]; then
        # 添加一个
        reprepro --ask-passphrase -Vb ${MIRROR_DIR} -C main -A ${ARCH} includedeb ${VERSION} ${DEBS_TARGET}
    fi
}

# 查看包列表
function repo_list() {
    TARGET=$1
    VERSION=$2
    ARCH=$3

    # 源发布目录
    MIRROR_DIR="${MIRROR_PREFIX}/${TARGET}"
    reprepro -Vb ${MIRROR_DIR} -A ${ARCH} list ${VERSION}
}

# 删除包
function repo_remove() {
    TARGET=$1
    VERSION=$2
    ARCH=$3
    PACKAGENAME=$4

    # 源发布目录
    MIRROR_DIR="${MIRROR_PREFIX}/${TARGET}"

    if [ "${PACKAGENAME}" = "all" ]; then
        # 批量删除
        reprepro -Vb ${MIRROR_DIR} -A ${ARCH} list ${VERSION} | awk '{print $2}' | xargs -i reprepro --ask-passphrase -Vb ${MIRROR_DIR} -C main -A ${ARCH} remove ${VERSION} {}
    else
        # 删除一个包
        reprepro --ask-passphrase -Vb ${MIRROR_DIR} -C main -A ${ARCH} remove ${VERSION} ${PACKAGENAME}
    fi
}

# 使用源
function repo_use() {
    # MIRROR_PATH="/media/${USER}/linux/mirrors/kylin/launchpad/kylin/mirror/archive.launchpad.dev"
    # if [ -d "${MIRROR_PATH}" ]; then
    #     python3 -m http.server -d "${MIRROR_PATH}"
    # fi

    # 使用 9000 端口
    python3 -m http.server -d ${MIRROR_PREFIX} 9000

    # 配置source list
    echo "deb http://172.29.157.20:9000/kylin v101 main" | sudo tee /etc/apt/sources.list.d/v101.list
    # 添加公钥
    wget -O - http://172.29.157.20:9000/kylin/public.key | sudo apt-key add -
    # 更新软件源
    sudo apt update
}

#################### 处理拷贝的x86桌面版deb包 ####################
function deal_x86_debs() {
    # 批量添加报如下错误

    # Warning: database 'v101|main|amd64' was modified but no index file was exported.
    # Changes will only be visible after the next 'export'!
    # There have been errors!

    # 类似的错误
    # No section given for 'sd24plq', skipping.
    # 1085 There have been errors!

    # 循环一个一个添加
    for i in $(find /data/desktop-deb/ -name *.deb); do
        echo $i
        reprepro --ask-passphrase -Vb ${MIRROR_CONF_DIR} -C main -A amd64 includedeb v101 $i
    done >>log.txt 2>&1

    # $ grep "No section" log.txt
    # No section given for 'sdtmepos', skipping.
    # No section given for 'hl3190cdwpdrv', skipping.
    # No section given for 'sd24plq', skipping.
    # No section given for 'sdzpl', skipping.
    # No section given for 'sd24poki', skipping.
    # No section given for 'sdcpcl', skipping.
    # No section given for 'hll9310cdwpdrv', skipping.
    # No section given for 'mfc9150cdnpdrv', skipping.

    # 刪除 沒有 Section: 字段的包
    for i in $(ls *.deb); do
        if ! dpkg-scanpackages $i | grep -q "Section:"; then
            echo "=== rm $i"
            rm $i
        fi
    done >>rm-deb.list

    # 针对以上包 使用 -S thirdpart 参数
    # 添加一个deb
    reprepro --ask-passphrase -Vb ${MIRROR_CONF_DIR} -S thirdpart -C main -A amd64 includedeb v101 desktop-deb/sdtmepos_1.0.0-1.2_amd64.deb
}
