#! /bin/bash
set -e

USERNAME=lnsjcy_ak01
DATA_DISK=/dev/sdb

function add_fstab() {
  DATA_PARTION=$1
  MOUNT_POINT=$2
  if [ -b ${DATA_PARTION} ]; then
    PARTION_UUID=$(blkid -s UUID -o value ${DATA_PARTION})

    if [ ! -d ${MOUNT_POINT} ]; then
      mkdir -p ${MOUNT_POINT}
      chown -R ${USERNAME}:${USERNAME} ${MOUNT_POINT}
      chmod -R 777 ${MOUNT_POINT}
    fi

    # 删除
    sed -i "/$(basename ${DATA_PARTION})/d" /etc/fstab
    sed -i "/${PARTION_UUID}/d" /etc/fstab

    # 添加
    echo "# ${DATA_PARTION}" >>/etc/fstab
    echo "UUID=${PARTION_UUID} ${MOUNT_POINT} ext4 rw 0 0" >>/etc/fstab

    mount ${DATA_PARTION} ${MOUNT_POINT}
    chown -R ${USERNAME}:${USERNAME} ${MOUNT_POINT}
    rm -rf "${MOUNT_POINT}/lost+found"

    if [[ ${MOUNT_POINT} =~ "/data" ]]; then
      chmod -R 777 ${MOUNT_POINT}
    else
      chmod -R 755 ${MOUNT_POINT}
    fi

    umount ${MOUNT_POINT}
  fi
}

# 平均分为两个分区
parted -s ${DATA_DISK} mktable gpt
parted -s ${DATA_DISK} mkpart primary ext4 1 50%
parted -s ${DATA_DISK} mkpart primary ext4 50% 100%
partprobe ${DATA_DISK}
sync

DATA_PARTION1=${DATA_DISK}1
DATA_PARTION2=${DATA_DISK}2

## 格式化
yes | mkfs -t ext4 -q -L data1 ${DATA_PARTION1} 2>/dev/null
yes | mkfs -t ext4 -q -L data2 ${DATA_PARTION2} 2>/dev/null

## 自动挂载
add_fstab "${DATA_PARTION1}" "/data1"
add_fstab "${DATA_PARTION2}" "/home/${USERNAME}/桌面"
