#! /bin/bash
set -e

USER=lnsjcy_ak01

chroot=chroot
ROOT=/target

function add_fstab() {
  DATA_PARTION=$1
  MOUNT_POINT=$2
  if [ -b ${DATA_PARTION} ]; then
    PARTION_UUID=$(blkid -s UUID -o value ${DATA_PARTION})
    echo "DATA_PARTION=${DATA_PARTION}, UUID=${PARTION_UUID}"

    if [ ! -d ${ROOT}${MOUNT_POINT} ]; then
      echo mkdir -p ${ROOT}${MOUNT_POINT}
      mkdir -p $ROOT/${MOUNT_POINT} || true

      $chroot $ROOT chmod -R 777 ${MOUNT_POINT} >/dev/null || true
      $chroot $ROOT chown $USER:$USER ${MOUNT_POINT} >/dev/null || true
    fi

    # 删除
    sed -i "/$(basename ${DATA_PARTION})/d" $ROOT/etc/fstab
    sed -i "/${PARTION_UUID}/d" $ROOT/etc/fstab

    # 添加
    echo "# ${DATA_PARTION}"
    echo "UUID=${PARTION_UUID} ${MOUNT_POINT} ext4 rw 0 0"
    echo "# ${DATA_PARTION}" >>$ROOT/etc/fstab
    echo "UUID=${PARTION_UUID} ${MOUNT_POINT} ext4 rw 0 0" >>$ROOT/etc/fstab

    if mountpoint -q ${ROOT}${MOUNT_POINT}; then
      umount ${ROOT}${MOUNT_POINT}
    fi

    mount ${DATA_PARTION} ${ROOT}${MOUNT_POINT}
    # rm -rf "${ROOT}${MOUNT_POINT}/lost+found"
    mv "${ROOT}${MOUNT_POINT}/lost+found" "${ROOT}${MOUNT_POINT}/.lost+found"

    $chroot $ROOT chown -R ${USER}:${USER} ${MOUNT_POINT}
    if [[ ${MOUNT_POINT} =~ "/data" ]]; then
      $chroot $ROOT chmod -R 777 ${MOUNT_POINT}
    else
      $chroot $ROOT chmod -R 755 ${MOUNT_POINT}
    fi

    umount ${ROOT}${MOUNT_POINT}
  else
    echo "${DATA_PARTION} not exist"
  fi
}

## 获取数据盘(取大于 800G 的硬盘)
DISKS=($(
  LANG=en
  fdisk -l | grep "Disk /dev" | awk -F '[ :]' '{ if($6 > 800*1024*1024*1024) print $2 }'
))
echo "DISKS=${DISKS[@]}"

DATA_DISK=${DISKS[0]}
echo "DATA_DISK=${DATA_DISK}"

# 平均分为两个分区
parted -s ${DATA_DISK} mktable gpt
parted -s ${DATA_DISK} mkpart primary ext4 1 50%
parted -s ${DATA_DISK} mkpart primary ext4 50% 100%
partprobe ${DATA_DISK}
sync

if [[ "${DATA_DISK}" =~ "/dev/nvme" ]]; then
  DATA_PARTION1=${DATA_DISK}p1
  DATA_PARTION2=${DATA_DISK}p2
elif [[ "${DATA_DISK}" =~ "/dev/sd" ]]; then
  DATA_PARTION1=${DATA_DISK}1
  DATA_PARTION2=${DATA_DISK}2
fi

## 格式化
yes | mkfs -t ext4 -q -L data1 ${DATA_PARTION1} 2>/dev/null || true
yes | mkfs -t ext4 -q -L data2 ${DATA_PARTION2} 2>/dev/null || true

## 自动挂载
echo add_fstab "${DATA_PARTION1}" "/data1"
add_fstab "${DATA_PARTION1}" "/data1"

echo add_fstab "${DATA_PARTION2}" "/home/${USER}/桌面"
add_fstab "${DATA_PARTION2}" "/home/${USER}/桌面"
