#!/bin/bash

#init base env



# build for rootfs

if [ `whoami` != "root" ];then
  echo "Please use root run the program"
  exit 1
fi

if [ -d rootfs ];then
  echo "rootfs dir exist"
else
  mkdir rootfs
fi

if [ $# -ne 4 ];then
  echo "========================================="
  echo "Usage:"
  echo "Use the program like:"
  echo -e "iso and isodir mode, default find squashfs file in /ISOPATH/casper/filesystem.squashfs."
  echo -e "\tiso mode:  #$0 USE_DEV iso ISOPATH"
  echo -e "\tLike: $0 /dev/sdb iso /NeoKylin.iso"
  echo -e "\tLike: $0 rootfs.img iso /NeoKylin.iso"
  echo "========================================="
  exit 1
fi

USE_DEV=$1
MODE_TYPE=$2
ISO_PATH=$3
VERSION=$4

ROOT_DEV=${USE_DEV}

# ISO mount directory
TEMPISODIR=`mktemp -t -d temp.XXXX`
TEMPSQUASHFSDIR=`mktemp -t -d temp.XXXX`
# root mount directory
TEMPROOTDEV=`mktemp -t -d temp.XXXX`

# architecture
# OS_ARCH="$(dpkg --print-architecture)" U
OS_ARCH=`arch`
if [ "`arch`" == "aarch64" ];then
	OS_ARCH="arm64"
fi

ISO_ARCH=''
IS_SERVER=0

# Return iso architecture: parameter iso directory
function_get_iso_arch(){
  ISO_ARCH="$(cat ${TMP_STRING_ISODIR}/.disk/info | awk -F ' - ' '{print $2}' | cut -d ' ' -f2)"
}

# Print success info
function_print_success_info(){
  echo -e "\\033[0;32m $1 \\033[0;39m"
}

# Print fail info
function_print_fail_info(){
  echo -e "\\033[0;31m $1 \\033[0;39m"
  function_clean_envirronment
  sync
  exit 1
}

# Print part info
function_part_info(){
        blkid | awk '{
                gsub( ":", "", $1 );
                dev=$1;

                for(i=2; i<=NF; i++ ){
                        if( index( $i, "UUID=" ) == 1 ){
                                gsub( "UUID=", "", $i );
                                gsub( "\"", "", $i );
                                uuid=$i;
                        }
                        else if( index( $i, "TYPE=") == 1 ){
                                gsub( "TYPE=", "", $i );
                                gsub( "\"", "", $i );
                                type=$i;
                        }
                        else
                                continue;
                }
                if( dev=="'$ROOT_DEV'" )      
                        printf "# /     was on %s \nUUID=%s    /    %s    errors=remount-ro    0    1\n",dev,uuid,type;
                else if( dev=="'$BOOT_DEV'" )
                        printf "# /boot/efi was on %s \n#UUID=%s    /boot/efi    %s    defaults    0    2\n",dev,uuid,type;
        }'

}

function_clean_envirronment(){
  echo "please wait for clean envirronment..."
  killall cupsd
  sync
  [ $(mount | grep -c $TEMPSQUASHFSDIR) -eq 1 ] && umount $TEMPSQUASHFSDIR
  [ $MODE_TYPE != "isodir" ] && { sleep 1; umount $TEMPISODIR; }
  [ $(mount | grep -c $TEMPROOTDEV) -eq 1 ] && umount $TEMPROOTDEV
  [ -d ${TEMPISODIR} ] && rm -rf ${TEMPISODIR}
  [ -d ${TEMPSQUASHFSDIR} ] && rm -rf ${TEMPSQUASHFSDIR}
#  [ -d ${TEMPROOTDEV} ] && rm -rf ${TEMPROOTDEV}
  sync
  echo "rubbish enviroment clean success"
  exit 0
}

# Format the hard disk
mkfs.ext4 -F $USE_DEV
[ $? -ne 0 ] && function_print_fail_info "mkfs.ext4 $USE_DEV error!" || function_print_success_info "mkfs.ext4 $USE_DEV ok!" 
sync

echo "current input type = ${MODE_TYPE}"
if [ $MODE_TYPE = "iso" ];then
  mount -o loop $ISO_PATH $TEMPISODIR
  [ $? -ne 0 ] && function_print_fail_info "mount iso $ISO_PATH error!" || function_print_success_info "mount iso $ISO_PATH ok!"

  mount -o loop $TEMPISODIR/casper/filesystem.squashfs $TEMPSQUASHFSDIR
  [ $? -ne 0 ] && function_print_fail_info "mount casper/filesystem.squashfs error!" || function_print_success_info "mount casper/filesystem.squashfs ok!"
elif  [ $MODE_TYPE = "isodir" ];then
  [ -d $TEMPISODIR ] && rm -rf $TEMPISODIR
  TEMPISODIR=$ISO_PATH
  
  mount -o loop $TEMPISODIR/casper/filesystem.squashfs $TEMPSQUASHFSDIR
  [ $? -ne 0 ] && function_print_fail_info "mount casper/filesystem.squashfs error!" || function_print_success_info "mount casper/filesystem.squashfs ok!"
elif  [ $MODE_TYPE = "squashfs" ];then
  mount -o loop $ISO_PATH $TEMPSQUASHFSDIR
  [ $? -ne 0 ] && function_print_fail_info "mount $ISO_PATH error!" || function_print_success_info "mount $ISO_PATH ok!"
else
  function_print_fail_info "you set mode error, please set iso or squashfs."
fi

function_get_iso_arch
# echo "iso arch: ${ISO_ARCH}"
function_print_success_info "os arch: ${OS_ARCH}"


#确保挂载时casper下有手动创建的roofs目录
mount $ROOT_DEV ./rootfs
[ $? -ne 0 ] && function_print_fail_info "mount $ROOT_DEV error!" || function_print_success_info "mount $ROOT_DEV ok!"

TEMPROOTDEV=./rootfs
echo "rsync filesystem.squashfs to $ROOT_DEV ......"
rsync -a ${TEMPSQUASHFSDIR}/ ${TEMPROOTDEV}
if [ $? -ne 0 ];then
  function_print_fail_info "rsync filesystem.squashfs to $ROOT_DEV  error!"
else
  function_print_success_info "rsync filesystem.squashfs to $ROOT_DEV  ok!"
fi

echo "***Show detail of rootfs after rsync***"
ls ${TEMPROOTDEV}

echo "Start to custom rootfs "
##password: perl -e 'print crypt("123123",q(kylin)),"\n"'
##password : Kylin!32888
chroot ${TEMPROOTDEV} /bin/bash -c "useradd -m  -d /home/kylin -s /bin/bash -p kyYFdQFAeVrPQ kylin"
#chroot ${TEMPROOTDEV} /bin/bash -c "echo "kylin:chenrong13632422963" | chpasswd"

#内部测试使用简易密码
chroot ${TEMPROOTDEV} /bin/bash -c "echo "kylin:qwe123" | chpasswd"

if [ $? -ne 0 ];then
    function_print_fail_info "chroot create user error!"
else
    function_print_success_info "chroot create user ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "addgroup kylin adm"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'addgroup kylin adm' error!"
else
    function_print_success_info "chroot run 'addgroup kylin adm' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "addgroup kylin sudo"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'addgroup kylin sudo' error!"
else
    function_print_success_info "chroot run 'addgroup kylin sudo' ok!"
fi


####test
<<!
chroot ${TEMPROOTDEV} /bin/bash -c "sed -i 's/launcher-location=firefox.desktop/launcher-location=chromium-browser.desktop/g' /usr/share/mate-panel/layouts/kylin.layout "
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'sed -i 's/launcher-location=firefox.desktop/launcher-location=chromium-browser.desktop' error!"
else
    function_print_success_info "chroot run 'sed -i 's/launcher-location=firefox.desktop/launcher-location=chromium-browser.desktop' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "sed -i 's/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build07-20210121.J1-arm64\"/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build01_DTS6004_01-20210122.J1-arm64\"/g' /etc/os-release"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'sed -i 's/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build07-20210121.J1-arm64\"/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build01_DTS6004_01-20210122.J1-arm64\"/g' /etc/os-release' error!"
else
    function_print_success_info "chroot run 'sed -i 's/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build07-20210121.J1-arm64\"/KYLIN-BUILDER=\"Kylin-4.0.2-desktop-sp2-terminal-Build01_DTS6004_01-20210122.J1-arm64\"/g' /etc/os-release' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "rm /usr/share/applications/firefox.desktop"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'rm /usr/share/applications/firefox.desktop' error!"
else
    function_print_success_info "chroot run 'rm /usr/share/applications/firefox.desktop' ok!"
fi

!
<<!
cp sogouimebs_2.4.0.3435_arm64.deb rootfs/
chroot ${TEMPROOTDEV} /bin/bash -c "dpkg -i /sogouimebs_2.4.0.3435_arm64.deb"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'dpkg -i /sogouimebs_2.4.0.3435_arm64.deb' error!"
else
    function_print_success_info "chroot run 'dpkg -i /sogouimebs_2.4.0.3435_arm64.deb' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "rm -f /sogouimebs_2.4.0.3435_arm64.deb"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'rm -f /sogouimebs_2.4.0.3435_arm64.deb' error!"
else
    function_print_success_info "chroot run 'rm -f /sogouimebs_2.4.0.3435_arm64.deb' ok!"
fi
!

<<!
##针对208的xdroid所做的改动
chroot ${TEMPROOTDEV} /bin/bash -c "mv /opt/interfaces /etc/network/interfaces"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'mv /opt/interfaces /etc/network/interfaces' error!"
else
    function_print_success_info "chroot run 'mv /opt/interfaces /etc/network/interfaces' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "chmod 777 /etc/network/interfaces"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'chmod 777 /etc/network/interfaces' error!"
else
    function_print_success_info "chroot run 'chmod 777 /etc/network/interfaces' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "mv /opt/rc.local /etc/rc.local"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'mv /opt/rc.local /etc/rc.local' error!"
else
    function_print_success_info "chroot run 'mv /opt/rc.local /etc/rc.local' ok!"
fi
!



<<!
# 麟卓
chroot ${TEMPROOTDEV} /bin/bash -c "chmod 777 /var/xdroid/common/data/xdroid/xDroidLed.conf"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'chmod 777 /var/xdroid/common/data/xdroid/xDroidLed.conf' error!"
else
    function_print_success_info "chroot run 'chmod 777 /var/xdroid/common/data/xdroid/xDroidLed.conf' ok!"
fi


##编译的权限
chroot ${TEMPROOTDEV} /bin/bash -c "chown -R kylin:kylin /var/xdroid/"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'chown -R kylin:kylin /var/xdroid/' error!"
else
    function_print_success_info "chroot run 'chown -R kylin:kylin /var/xdroid/' ok!"
fi
!

##home的权限
chroot ${TEMPROOTDEV} /bin/bash -c "chown -R kylin:kylin /home/kylin/"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'chown -R kylin:kylin /home/kylin/' error!"
else
    function_print_success_info "chroot run 'chown -R kylin:kylin /home/kylin/' ok!"
fi


##208 change end 
chroot ${TEMPROOTDEV} /bin/bash -c "addgroup kylin lpadmin"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'addgroup kylin lpadmin' error!"
else
    function_print_success_info "chroot run 'addgroup kylin lpadmin' ok!"
fi

chroot ${TEMPROOTDEV} /bin/bash -c "addgroup kylin video"
if [ $? -ne 0 ];then
    function_print_fail_info "chroot run 'addgroup kylin video' error!"
else
    function_print_success_info "chroot run 'addgroup kylin video' ok!"
fi

if [ -f ${TEMPISODIR}/.kylin-post-actions ];then
    cp -a ${TEMPISODIR}/.kylin-post-actions  ${TEMPROOTDEV}/bin/
    if [ $? -ne 0 ];then
        function_print_fail_info "copy /.kylin-post-actions from iso to target /bin/  error!"
    else
        function_print_success_info "copy /.kylin-post-actions from iso to target /bin/ ok!"
    fi
fi
if [ -f ${TEMPROOTDEV}/bin/.kylin-post-actions ];then
    echo "Will do kylin-post-actions ...... "
    chroot ${TEMPROOTDEV} /bin/bash -c "/bin/bash /bin/.kylin-post-actions"
    if [ $? -ne 0 ];then
        function_print_fail_info "chroot run '/bin/bash /bin/.kylin-post-actions' error!"
    else
        function_print_success_info "chroot run '/bin/bash /bin/.kylin-post-actions' ok!"
    fi
fi

#因为搜狗输入法的BUG，需手动帮其做配置，非通用性功能
if [ -d ${TEMPROOTDEV}/usr/share/sogouimebs ];then
	echo "do some config for sogou input"
	echo "make sogou autostart at a front postion"
	chroot ${TEMPROOTDEV} /bin/bash -c "ln -sf /etc/xdg/autostart/sogouimebs.desktop /etc/xdg/autostart/a-sogou.desktop"
	chroot ${TEMPROOTDEV} /bin/bash -c "ln -sf /etc/xdg/autostart/sogouimebs-watchdog.desktop /etc/xdg/autostart/a-sogou-watchdog.desktop"
	echo "set sogou default input delay timeout"
	chroot ${TEMPROOTDEV} /bin/bash -c "sed -i 's/auto_time=\"200\"/auto_time=\"800\"/g' /usr/share/sogouimebs/Resources/PC/keyboard_phev.xml"
	chroot ${TEMPROOTDEV} /bin/bash -c "sed -i 's/auto_time=\"200\"/auto_time=\"800\"/g' /usr/share/sogouimebs/Resources/PC/keyboard_phev1024.xml"
fi
#834修改 by dengml
chroot ${TEMPROOTDEV} /bin/bash -c "chmod +x /usr/bin/fingerpaint"
chroot ${TEMPROOTDEV} /bin/bash -c "chmod +x /usr/bin/touchscreen.sh"

#将版本号写入os-release文件
chroot ${TEMPROOTDEV} /bin/bash -c "sed  -i '2c VERSION=\"${VERSION}\" '  /etc/os-release"
cat  ${TEMPROOTDEV}/etc/os-release

#强行确保HOME目录权限正常
chroot ${TEMPROOTDEV} /bin/bash -c "chown -R kylin:kylin /home/kylin"

if [ ${MODE_TYPE} = "iso" ];then
  sq_dir="${TEMPISODIR}/casper/"
elif  [ ${MODE_TYPE} = "isodir" ];then
  sq_dir="${TEMPISODIR}/casper/"
elif  [ ${MODE_TYPE} = "squashfs" ];then
  sq_dir=`dirname ${ISO_PATH}`
fi

if [ -s $sq_dir/filesystem.manifest-remove ];then
  echo "============================>>>"
  echo "Now delete some pkgs..."
  while read line
  do
    chroot ${TEMPROOTDEV} /bin/bash -c "apt-get -y purge $line &>/dev/null"
  done < $sq_dir/filesystem.manifest-remove

  chroot ${TEMPROOTDEV} /bin/bash -c "dpkg-query -W --showformat='${Package}\n' | sort > /new-debs"
  tempfile1=`mktemp temp.XXXX`
  tempfile2=`mktemp temp.XXXX`
  cat ${TEMPROOTDEV}/new-debs | sed 's/ //g' | sed '/^$/d' | sort > $tempfile1
  cat $sq_dir/filesystem.manifest-remove | sed 's/ //g' | sed '/^$/d' | sort > $tempfile2
  rm -rf ${TEMPROOTDEV}/new-debs
  err_del_pkg=`comm -12 $tempfile1 $tempfile2`
  if [ -n "$err_del_pkg" ];then
    function_print_fail_info "Some pkgs delete error:---->"
    function_print_fail_info "$err_del_pkg"
    function_print_success_info "Some pkgs delete error end"
  fi
  rm -rf $tempfile1
  rm -rf $tempfile2
fi

function_print_success_info "ALL DONE."
function_clean_envirronment

