#!/bin/bash

set -e

umount -l /media/*/* >/dev/null 2>&1 || true

# 分区
disk=/dev/sda
part_efi="${disk}1"
part_boot="${disk}2"
part_root="${disk}3"

efi=256
boot=1024
root=20480

parted -s "${disk}" mktable gpt

## efi
start=1
end=$((efi + 1))
parted -s "${disk}" mkpart "EFI" fat32 1MiB "${end}"MiB

## boot
start=${end}
end=$((end + boot))
parted -s "${disk}" mkpart "boot" ext4 "${start}"MiB "${end}"MiB

## root
start=${end}
end=$((end + root))
parted -s "${disk}" mkpart primary ext4 "${start}"MiB "${end}"MiB

partprobe "${disk}"
sync

umount -l /media/*/* >/dev/null 2>&1 || true

## format
yes | mkfs.vfat -n ESP "${part_efi}"
yes | mkfs.ext4 -Fq -L SYSBOOT "${part_boot}"
yes | mkfs.ext4 -Fq -L SYSROOT "${part_root}"


umount -l /media/*/* >/dev/null 2>&1 || true

# mountfs
mkdir -p /target/
mount "${part_root}" /target

# 拷贝文件系统
if ! mountpoint -q /rofs; then
    SYSIMG=/home/zhen/images/qemuarm64/isar-image-base-kylin-v101-qemuarm64.ext4.img
    mkdir -p /rofs
    mount "${SYSIMG}" /rofs
fi
echo "rsync -aHAX /rofs/ /target"
rsync -aHAX /rofs/ /target

# tar xvf xxx.tar.gz -C /target

sync

mkdir -p /target/boot
mount "${part_boot}" /target/boot
mkdir -p /target/boot/efi
mount "${part_efi}" /target/boot/efi

mount --bind /dev /target/dev
mount --bind /run /target/run
mount -vt devpts devpts /target/dev/pts -o gid=5,mode=620
mount -vt proc proc /target/proc
mount -vt sysfs sysfs /target/sys

# # ********** into chroot **********
# sudo chroot /target
# apt update
# apt install -y grub2-common grub-efi-arm64 efibootmgr
# grub-install

# ## timeout -> 9s
# sed -i '/GRUB_DEFAULT/ {s/0/9/}' /etc/default/grub

# root_uuid=$(lsblk -f | grep SYSROOT | awk '{print $4}')
# custom_grub_cfg=/etc/grub.d/40_custom
# echo "root_uuid=${root_uuid}"

# cat >>$custom_grub_cfg <<EOF
# menuentry 'kylin v101 arm64' --class gnu-linux --class gnu --class os \$menuentry_id_option 'gnulinux-simple-$root_uuid' {
#     recordfail
#     load_video
#     gfxmode \$linux_gfx_mode

#     insmod gzio
#     insmod part_gpt
#     insmod ext2

#     search --no-floppy --fs-uuid --set=root $root_uuid
#     linux   /boot/vmlinuz-5.4.0-26-generic root=UUID=${root_uuid} ro  quiet splash \$vt_handoff
#     initrd  /boot/initrd.img-5.4.0-26-generic
# }
# EOF

# update-grub

# # ********** end chroot **********

# umount -l /target/dev/
# umount /target/sys/
# umount /target/run/
# umount /target/proc/
# umount -l /target
